# SPDX-License-Identifier: LGPL-2.1-only
#

from typing import Any, Callable, Dict, FrozenSet, Iterable, Iterator, List, NamedTuple, \
    NoReturn, Optional, Set, Tuple, Union

import enum
import ipaddress
import setools.exception
import weakref

AnyConstraint = Union["Constraint", "Validatetrans"]
AnyDefault = Union["Default", "DefaultRange"]
AnyRBACRule = Union["RoleAllow", "RoleTransition"]
AnyTERule = Union["AVRule", "AVRuleXperm", "TERule", "FileNameTERule"]
SELINUX_SIDNAMES: Any
TypeOrAttr = Union["Type", "TypeAttribute"]
XEN_SIDNAMES: Any

def lookup_boolean_name_sub(name: str) -> str: ...

class AVRule(BaseTERule):
    default: NoReturn = ...
    perms: Set[str] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def derive_expanded(self, *args, **kwargs) -> AVRule: ...
    def expand(self, *args, **kwargs) -> Iterable[AVRule]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class AVRuleXperm(BaseTERule):
    default: NoReturn = ...
    perms: IoctlSet = ...
    xperm_type: str = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self, *args, **kwargs) -> Iterable[AVRuleXperm]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BaseConstraint(PolicyObject):
    expression: ConstraintExpression = ...
    perms: FrozenSet[str] = ...
    ruletype: ConstraintRuletype = ...
    tclass: ObjClass = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BaseMLSLevel(PolicyObject):
    sensitivity: Sensitivity = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def categories(self) -> Iterable[Category]: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BaseTERule(PolicyRule):
    conditional: Conditional = ...
    conditional_block: bool = ...
    filename: str = ...
    ruletype: TERuletype = ...
    source: TypeOrAttr = ...
    tclass: ObjClass = ...
    target: TypeOrAttr = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def enabled(self, **kwargs) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BaseType(PolicySymbol):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def aliases(self) -> Iterable[str]: ...
    def attributes(self) -> Iterable[BaseType]: ...
    def expand(self) -> Iterable[BaseType]: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Boolean(PolicySymbol):
    state: bool = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BooleanHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Boolean: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Bounds(PolicyObject):
    child: Type = ...
    parent: Type = ...
    ruletype: BoundsRuletype = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class BoundsRuletype(PolicyEnum):
    typebounds: int = ...

class Category(PolicySymbol):
    _value: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def aliases(self, *args, **kwargs) -> Iterable[str]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class CategoryEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Category: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class CategoryHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Category: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Common(PolicySymbol):
    perms: FrozenSet[str] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __contains__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class CommonHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Common: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Conditional(PolicyObject):
    booleans: FrozenSet[Boolean] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def evaluate(self, **kwargs) -> bool: ...
    def expression(self) -> Iterable[Union[ConditionalOperator, str]]: ...
    def false_rules(self, *args, **kwargs) -> Iterable[AnyTERule]: ...
    def statement(self) -> str: ...
    def true_rules(self, *args, **kwargs) -> Iterable[AnyTERule]: ...
    def truth_table(self) -> List[TruthTableRow]: ...
    def __contains__(self, other) -> bool: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConditionalExprIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Union[ConditionalOperator, Boolean]: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConditionalIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Conditional: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConditionalOperator(PolicyObject):
    _cond_expr_val_to_details: Any = ...
    evaluate: Callable = ...
    precedence: int = ...
    unary: bool = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self, *args, **kwargs) -> NoReturn: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConditionalTERuleIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def ruletype_count(self, *args, **kwargs) -> Any: ...
    def __len__(self) -> int: ...
    def __next__(self) -> AnyTERule: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Constraint(BaseConstraint):
    perms: FrozenSet[str] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConstraintExprIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> ConstraintExprNode: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConstraintExprNode(PolicyObject):
    _expr_op_to_text: Any = ...
    _expr_type_to_text: Any = ...
    _role_syms: Any = ...
    _sym_to_text: Any = ...
    _type_syms: Any = ...
    _user_syms: Any = ...
    names: Union[FrozenSet[TypeOrAttr], FrozenSet[Role], FrozenSet[User]] = ...
    symbol_type: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> NoReturn: ...
    def __contains__(self, other) -> bool: ...
    def __getitem__(self, index) -> Union[str, FrozenSet[TypeOrAttr], FrozenSet[Role], FrozenSet[User]]: ...
    def __iter__(self) -> Iterable[Union[str, FrozenSet[TypeOrAttr], FrozenSet[Role], FrozenSet[User]]]: ...
    def __len__(self) -> int: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConstraintExpression(PolicyObject):
    mls: bool = ...
    roles: FrozenSet[Role] = ...
    types: FrozenSet[Type] = ...
    users: FrozenSet[User] = ...
    __hash__: Any = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def infix(self, *args, **kwargs) -> Union[str, FrozenSet[TypeOrAttr], FrozenSet[Role], FrozenSet[User]]: ...
    def statement(self) -> NoReturn: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __getitem__(self, index) -> Union[str, FrozenSet[TypeOrAttr], FrozenSet[Role], FrozenSet[User]]: ...
    def __gt__(self, other) -> bool: ...
    def __iter__(self) -> Iterator[Union[FrozenSet[Role], FrozenSet[Type], FrozenSet[User], str]]: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConstraintIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Constraint: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ConstraintRuletype(PolicyEnum):
    constrain: int = ...
    mlsconstrain: int = ...
    mlsvalidatetrans: int = ...
    validatetrans: int = ...

class ConstraintUseError(setools.exception.SymbolUseError): ...

class Context(PolicyObject):
    range_: Range = ...
    role: Role = ...
    type_: Type = ...
    user: User = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Default(PolicyObject):
    default: DefaultValue = ...
    ruletype: DefaultRuletype = ...
    tclass: ObjClass = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class DefaultRange(Default):
    default_range: DefaultRangeValue = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class DefaultRangeValue(PolicyEnum):
    high: int = ...
    low: int = ...
    low_high: int = ...
    @classmethod
    def from_default_range(self, range: Optional[int]) -> Optional[DefaultRangeValue]: ...

class DefaultRuletype(PolicyEnum):
    default_range: int = ...
    default_role: int = ...
    default_type: int = ...
    default_user: int = ...

class DefaultValue(PolicyEnum):
    glblub: int = ...
    source: int = ...
    target: int = ...
    @classmethod
    def from_default_range(self, *args, **kwargs) -> Any: ...

class Devicetreecon(Ocontext):
    path: str = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class DevicetreeconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Devicetreecon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class EbitmapIterator(PolicyIterator):
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> PolicyObject: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class FSUse(Ocontext):
    fs: str = ...
    ruletype: FSUseRuletype = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class FSUseIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> FSUse: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class FSUseRuletype(PolicyEnum):
    fs_use_task: int = ...
    fs_use_trans: int = ...
    fs_use_xattr: int = ...

class FileNameTERule(BaseTERule):
    default: Type = ...
    filename: str = ...
    perms: NoReturn = ...
    ruletype: TERuletype = ...
    source: TypeOrAttr = ...
    tclass: ObjClass = ...
    target: TypeOrAttr = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self, *args, **kwargs) -> Iterable[FileNameTERule]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class FileNameTERuleIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> FileNameTERule: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class GenfsFiletype(int):
    _filetype_to_text: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Genfscon(Ocontext):
    _sclass_to_stat: Any = ...
    filetype: GenfsFiletype = ...
    fs: str = ...
    path: str = ...
    tclass: ObjClass = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class GenfsconIterator:
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __iter__(self) -> Iterable[Genfscon]: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Genfscon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class GenfsconOcontextIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Ocontext: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class HandleUnknown(PolicyEnum):
    allow: int = ...
    deny: int = ...
    reject: int = ...

class HashtabIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Any: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Ibendportcon(Ocontext):
    name: str = ...
    port: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IbendportconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Ibendportcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Ibpkeycon(Ocontext):
    pkeys: IbpkeyconRange = ...
    subnet_prefix: ipaddress.IPv6Address = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IbpkeyconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Ibpkeycon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IbpkeyconRange(Tuple[int, int]):
    _asdict: Any = ...
    _field_defaults: Any = ...
    _fields: Any = ...
    _fields_defaults: Any = ...
    _replace: Any = ...
    high: int = ...
    low: int = ...
    __getnewargs__: Any = ...
    __slots__: Any = ...
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def _make(self, *args, **kwargs) -> Any: ...

class InitialSID(Ocontext):
    name: str = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class InitialSIDIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> InitialSID: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class InvalidBoolean(setools.exception.InvalidSymbol): ...

class InvalidBoundsType(setools.exception.InvalidSymbol): ...

class InvalidCategory(setools.exception.InvalidSymbol): ...

class InvalidClass(setools.exception.InvalidSymbol): ...

class InvalidCommon(setools.exception.InvalidSymbol): ...

class InvalidConstraintType(setools.exception.InvalidSymbol): ...

class InvalidDefaultRange(setools.exception.InvalidSymbol): ...

class InvalidDefaultType(setools.exception.InvalidRuleType): ...

class InvalidDefaultValue(setools.exception.InvalidSymbol): ...

class InvalidFSUseType(setools.exception.InvalidRuleType): ...

class InvalidInitialSid(setools.exception.InvalidSymbol): ...

class InvalidLevel(setools.exception.InvalidSymbol): ...

class InvalidLevelDecl(setools.exception.InvalidSymbol): ...

class InvalidMLSRuleType(setools.exception.InvalidRuleType): ...

class InvalidPolicy(ValueError, setools.exception.PolicyrepException): ...

class InvalidRBACRuleType(setools.exception.InvalidRuleType): ...

class InvalidRange(setools.exception.InvalidSymbol): ...

class InvalidRole(setools.exception.InvalidSymbol): ...

class InvalidRuleType(setools.exception.InvalidSymbol): ...

class InvalidSensitivity(setools.exception.InvalidSymbol): ...

class InvalidTERuleType(setools.exception.InvalidRuleType): ...

class InvalidType(setools.exception.InvalidSymbol): ...

class InvalidUser(setools.exception.InvalidSymbol): ...

class IoctlSet(FrozenSet[int]):
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def ranges(self) -> int: ...
    def __format__(self, *args, **kwargs) -> str: ...
    def __reduce_cython__(self, *args, **kwargs) -> Any: ...
    def __setstate_cython__(self, *args, **kwargs) -> Any: ...

class Iomemcon(Ocontext):
    addr: IomemconRange = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IomemconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Iomemcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IomemconRange(Tuple[int, int]):
    _asdict: Any = ...
    _field_defaults: Any = ...
    _fields: Any = ...
    _fields_defaults: Any = ...
    _replace: Any = ...
    high: int = ...
    low: int = ...
    __getnewargs__: Any = ...
    __slots__: Any = ...
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def _make(self, *args, **kwargs) -> Any: ...

class Ioportcon(Ocontext):
    ports: IoportconRange = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IoportconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Ioportcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class IoportconRange(Tuple[int, int]):
    _asdict: Any = ...
    _field_defaults: Any = ...
    _fields: Any = ...
    _fields_defaults: Any = ...
    _replace: Any = ...
    high: int = ...
    low: int = ...
    __getnewargs__: Any = ...
    __slots__: Any = ...
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def _make(self, *args, **kwargs) -> Any: ...

class Level(BaseMLSLevel):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> NoReturn: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __rxor__(self, other) -> bool: ...
    def __setstate__(self, state) -> Any: ...
    def __xor__(self, other) -> bool: ...

class LevelDecl(BaseMLSLevel):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class LevelDeclHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> LevelDecl: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class LowLevelPolicyError(ValueError, setools.exception.PolicyrepException): ...

class MLSDisabled(setools.exception.PolicyrepException): ...

class MLSRule(PolicyRule):
    default: Range = ...
    origin: MLSRule = ...
    ruletype: MLSRuletype = ...
    source: TypeOrAttr = ...
    tclass: ObjClass = ...
    target: TypeOrAttr = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self) -> Iterable[MLSRule]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class MLSRuleIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> MLSRuletype: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class MLSRuletype(PolicyEnum):
    range_transition: int = ...

class Netifcon(Ocontext):
    netif: str = ...
    packet: Context = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class NetifconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Netifcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class NoCommon(AttributeError, setools.exception.PolicyrepException): ...

class NoDefaults(setools.exception.InvalidSymbol): ...

class NoStatement(setools.exception.SymbolUseError): ...

class Nodecon(Ocontext):
    ip_version: NodeconIPVersion = ...
    network: Union[ipaddress.IPv4Network, ipaddress.IPv6Network] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class NodeconIPVersion(PolicyEnum):
    ipv4: int = ...
    ipv6: int = ...

class NodeconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Nodecon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ObjClass(PolicySymbol):
    common: Common = ...
    perms: FrozenSet[str] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def constraints(self, *args, **kwargs) -> Iterable[Constraint]: ...
    def defaults(self, *args, **kwargs) -> Iterable[AnyDefault]: ...
    def statement(self) -> str: ...
    def validatetrans(self, *args, **kwargs) -> Iterable[Validatetrans]: ...
    def __contains__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ObjClassHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> ObjClass: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Ocontext(PolicyObject):
    context: Context = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class OcontextIterator(PolicyIterator):
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Ocontext: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Pcidevicecon(Ocontext):
    device: str = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PcideviceconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Pcidevicecon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PermissionVectorIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Pirqcon(Ocontext):
    irq: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PirqconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Pirqcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyCapability(PolicySymbol):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyCapabilityIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> PolicyCapability: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyEnum(enum.Enum):
    @classmethod
    def lookup(self, value) -> Any: ...
    def __eq__(self, other) -> bool: ...
    def __format__(self, *args, **kwargs) -> Any: ...
    def __hash__(self) -> int: ...

class PolicyIterator:
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Any: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyObject:
    policy: SELinuxPolicy = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __copy__(self) -> PolicyObject: ...
    def __deepcopy__(self) -> PolicyObject: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyRule(PolicyObject):
    conditional: Conditional = ...
    conditional_block: bool = ...
    extended: bool = ...
    origin: PolicyRule = ...
    ruletype: Any = ...
    source: PolicySymbol = ...
    target: PolicySymbol = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def enabled(self, **kwargs) -> bool: ...
    def expand(self) -> Any: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicySymbol(PolicyObject):
    name: str = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PolicyTarget(PolicyEnum):
    selinux: int = ...
    xen: int = ...

class Portcon(Ocontext):
    ports: PortconRange = ...
    protocol: PortconProtocol = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PortconIterator(OcontextIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Portcon: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class PortconProtocol(PolicyEnum):
    dccp: int = ...
    sctp: int = ...
    tcp: int = ...
    udp: int = ...

class PortconRange(Tuple[int, int]):
    _asdict: Any = ...
    _field_defaults: Any = ...
    _fields: Any = ...
    _fields_defaults: Any = ...
    _replace: Any = ...
    high: int = ...
    low: int = ...
    __getnewargs__: Any = ...
    __slots__: Any = ...
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def _make(self, *args, **kwargs) -> Any: ...

class RBACRuletype(PolicyEnum):
    allow: int = ...
    role_transition: int = ...

class Range(PolicyObject):
    high: Level = ...
    low: Level = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __contains__(self, other) -> bool: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Role(PolicySymbol):
    dominated_roles: FrozenSet[Role] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self) -> Iterable[Role]: ...
    def statement(self) -> str: ...
    def types(self) -> Iterable[Type]: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleAllow(PolicyRule):
    default: NoReturn = ...
    ruletype: RBACRuletype = ...
    source: Role = ...
    tclass: NoReturn = ...
    target: Role = ...

    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self) -> Iterable[RoleAllow]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleAllowIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> RoleAllow: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Any: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Role: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleTransition(PolicyRule):
    default: Role = ...
    ruletype: RBACRuletype = ...
    source: Role = ...
    tclass: ObjClass = ...
    target: TypeOrAttr = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self) -> Iterable[RoleTransition]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RoleTransitionIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> RoleTransition: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class RuleNotConditional(AttributeError, setools.exception.PolicyrepException): ...

class RuleUseError(setools.exception.SymbolUseError): ...

class SELinuxPolicy:
    allow_count: int = ...
    allowxperm_count: int = ...
    auditallow_count: int = ...
    auditallowxperm_count: int = ...
    boolean_count: int = ...
    category_count: int = ...
    class_count: int = ...
    common_count: int = ...
    conditional_count: int = ...
    constraint_count: int = ...
    default_count: int = ...
    devicetreecon_count: int = ...
    dontaudit_count: int = ...
    dontauditxperm_count: int = ...
    fs_use_count: int = ...
    genfscon_count: int = ...
    handle_unknown: HandleUnknown = ...
    ibendportcon_count: int = ...
    ibpkeycon_count: int = ...
    initialsids_count: int = ...
    iomemcon_count: int = ...
    ioportcon_count: int = ...
    level_count: int = ...
    mls: bool = ...
    mlsconstraint_count: int = ...
    mlsvalidatetrans_count: int = ...
    netifcon_count: int = ...
    neverallow_count: int = ...
    neverallowxperm_count: int = ...
    nodecon_count: int = ...
    path: str = ...
    pcidevicecon_count: int = ...
    permission_count: int = ...
    permissives_count: int = ...
    pirqcon_count: int = ...
    polcap_count: int = ...
    portcon_count: int = ...
    range_transition_count: int = ...
    role_allow_count: int = ...
    role_count: int = ...
    role_transition_count: int = ...
    target_platform: PolicyTarget = ...
    type_attribute_count: int = ...
    type_change_count: int = ...
    type_count: int = ...
    type_member_count: int = ...
    type_transition_count: int = ...
    typebounds_count: int = ...
    user_count: int = ...
    validatetrans_count: int = ...
    version: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, policyfile: Optional[str] = None) -> None: ...
    def bools(self) -> Iterable[Boolean]: ...
    def bounds(self) -> Iterable[Bounds]: ...
    def categories(self) -> Iterable[Category]: ...
    def classes(self) -> Iterable[ObjClass]: ...
    def commons(self) -> Iterable[Common]: ...
    def conditionals(self) -> Iterable[Conditional]: ...
    def constraints(regularandMLS) -> Iterable[AnyConstraint]: ...
    def defaults(self) -> Iterable[AnyDefault]: ...
    def devicetreecons(self) -> Iterable[Devicetreecon]: ...
    def fs_uses(self) -> Iterable[FSUse]: ...
    def genfscons(self) -> Iterable[Genfscon]: ...
    def ibendportcons(self) -> Iterable[Ibendportcon]: ...
    def ibpkeycons(self) -> Iterable[Ibpkeycon]: ...
    def initialsids(self) -> Iterable[InitialSID]: ...
    def iomemcons(self) -> Iterable[Iomemcon]: ...
    def ioportcons(self) -> Iterable[Ioportcon]: ...
    def levels(self) -> Iterable[Level]: ...
    def lookup_boolean(self, name: Union[Boolean, str]) -> Boolean: ...
    def lookup_category(self, name: Union[Category, str], deref: bool = True) -> Category: ...
    def lookup_class(self, name: Union[ObjClass, str]) -> ObjClass: ...
    def lookup_common(self, name: Union[Common, str]) -> Common: ...
    def lookup_initialsid(self, name: Union[InitialSID, str]) -> InitialSID: ...
    def lookup_level(self, name: Union[Level, str]) -> Level: ...
    def lookup_range(self, name: Union[Range, str]) -> Range: ...
    def lookup_role(self, name: Union[Role, str]) -> Role: ...
    def lookup_sensitivity(self, name: Union[Sensitivity, str]) -> Sensitivity: ...
    def lookup_type(self, name: Union[Type, str], deref: bool = True) -> Type: ...
    def lookup_type_or_attr(self, name: Union[TypeOrAttr, str], deref: bool = True) -> TypeOrAttr: ...
    def lookup_typeattr(self, name: Union[TypeAttribute, str]) -> TypeAttribute: ...
    def lookup_user(self, name: Union[User, str]) -> User: ...
    def mlsrules(self) -> Iterable[MLSRule]: ...
    def netifcons(self) -> Iterable[Netifcon]: ...
    def nodecons(self) -> Iterable[Nodecon]: ...
    def pcidevicecons(self) -> Iterable[Pcidevicecon]: ...
    def pirqcons(self) -> Iterable[Pirqcon]: ...
    def polcaps(self) -> Iterable[PolicyCapability]: ...
    def portcons(self) -> Iterable[Portcon]: ...
    def rbacrules(self) -> Iterable[AnyRBACRule]: ...
    def roles(self) -> Iterable[Role]: ...
    def sensitivities(self) -> Iterable[Sensitivity]: ...
    def terules(self) -> Iterable[AnyTERule]: ...
    def typeattributes(self) -> Iterable[TypeAttribute]: ...
    def types(self) -> Iterable[Type]: ...
    def users(self) -> Iterable[User]: ...
    def __copy__(self) -> SELinuxPolicy: ...
    def __deepcopy__(self) -> SELinuxPolicy: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Sensitivity(PolicySymbol):
    _value: int = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def aliases(self, *args, **kwargs) -> Any: ...
    def level_decl(self, *args, **kwargs) -> Any: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class SensitivityHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Sensitivity: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class SymbolUseError(AttributeError, setools.exception.PolicyrepException): ...

class TERule(BaseTERule):
    default: Type = ...
    perms: NoReturn = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def expand(self) -> Iterable[TERule]: ...
    def statement(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TERuleIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def ruletype_count(self, *args, **kwargs) -> Any: ...
    def __len__(self) -> int: ...
    def __next__(self) -> TERule: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TERuleNoFilename(AttributeError, setools.exception.PolicyrepException): ...

class TERuletype(PolicyEnum):
    allow: int = ...
    allowxperm: int = ...
    auditallow: int = ...
    auditallowxperm: int = ...
    dontaudit: int = ...
    dontauditxperm: int = ...
    neverallow: int = ...
    neverallowxperm: int = ...
    type_change: int = ...
    type_member: int = ...
    type_transition: int = ...

class TruthTableRow(NamedTuple):
    values: Dict[str, bool]
    result: bool

class Type(BaseType):
    ispermissive: bool = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def aliases(self) -> Iterable[str]: ...
    def attributes(self) -> Iterable[TypeAttribute]: ...
    def expand(self) -> Iterable[Type]: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeAttribute(BaseType):
    ispermissive: bool = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def aliases(self) -> Iterable[str]: ...
    def attributes(self) -> Iterable[TypeAttribute]: ...
    def expand(self) -> Iterable[Type]: ...
    def statement(self) -> str: ...
    def __contains__(self, other) -> bool: ...
    def __iter__(self) -> Iterable[TypeAttribute]: ...
    def __len__(self) -> int: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeAttributeEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> TypeAttribute: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeAttributeHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> TypeAttribute: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> Type: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Type: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeOrAttributeEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> TypeOrAttr: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class TypeboundsIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Bounds: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class User(PolicySymbol):
    mls_level: Level = ...
    mls_range: Range = ...
    roles: FrozenSet[Role] = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class UserEbitmapIterator(EbitmapIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> User: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class UserHashtabIterator(HashtabIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def __next__(self) -> User: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class Validatetrans(BaseConstraint):
    perms: NoReturn = ...
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def statement(self) -> str: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class ValidatetransIterator(PolicyIterator):
    __pyx_vtable__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def __len__(self) -> int: ...
    def __next__(self) -> Validatetrans: ...
    def __reduce__(self) -> Any: ...
    def __setstate__(self, state) -> Any: ...

class WeakKeyDefaultDict(weakref.WeakKeyDictionary):
    _abc_impl: Any = ...
    __abstractmethods__: Any = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def __getitem__(self, index) -> Any: ...
    def __missing__(self, *args, **kwargs) -> Any: ...



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _semanage
else:
    import _semanage

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


SEMANAGE_MSG_ERR = _semanage.SEMANAGE_MSG_ERR
SEMANAGE_MSG_WARN = _semanage.SEMANAGE_MSG_WARN
SEMANAGE_MSG_INFO = _semanage.SEMANAGE_MSG_INFO

def semanage_msg_get_level(handle):
    return _semanage.semanage_msg_get_level(handle)

def semanage_msg_get_channel(handle):
    return _semanage.semanage_msg_get_channel(handle)

def semanage_msg_get_fname(handle):
    return _semanage.semanage_msg_get_fname(handle)

def semanage_msg_set_callback(handle, msg_callback, msg_callback_arg):
    return _semanage.semanage_msg_set_callback(handle, msg_callback, msg_callback_arg)

def semanage_handle_create():
    return _semanage.semanage_handle_create()

def semanage_handle_destroy(arg1):
    return _semanage.semanage_handle_destroy(arg1)
SEMANAGE_CON_INVALID = _semanage.SEMANAGE_CON_INVALID
SEMANAGE_CON_DIRECT = _semanage.SEMANAGE_CON_DIRECT
SEMANAGE_CON_POLSERV_LOCAL = _semanage.SEMANAGE_CON_POLSERV_LOCAL
SEMANAGE_CON_POLSERV_REMOTE = _semanage.SEMANAGE_CON_POLSERV_REMOTE

def semanage_select_store(handle, path, storetype):
    return _semanage.semanage_select_store(handle, path, storetype)

def semanage_reload_policy(handle):
    return _semanage.semanage_reload_policy(handle)

def semanage_set_reload(handle, do_reload):
    return _semanage.semanage_set_reload(handle, do_reload)

def semanage_set_rebuild(handle, do_rebuild):
    return _semanage.semanage_set_rebuild(handle, do_rebuild)

def semanage_set_check_ext_changes(handle, do_check):
    return _semanage.semanage_set_check_ext_changes(handle, do_check)

def semanage_get_hll_compiler_path(sh, lang_ext):
    return _semanage.semanage_get_hll_compiler_path(sh, lang_ext)

def semanage_set_create_store(handle, create_store):
    return _semanage.semanage_set_create_store(handle, create_store)

def semanage_get_disable_dontaudit(handle):
    return _semanage.semanage_get_disable_dontaudit(handle)

def semanage_set_disable_dontaudit(handle, disable_dontaudit):
    return _semanage.semanage_set_disable_dontaudit(handle, disable_dontaudit)

def semanage_set_check_contexts(sh, do_check_contexts):
    return _semanage.semanage_set_check_contexts(sh, do_check_contexts)

def semanage_get_default_priority(sh):
    return _semanage.semanage_get_default_priority(sh)

def semanage_set_default_priority(sh, priority):
    return _semanage.semanage_set_default_priority(sh, priority)

def semanage_is_managed(arg1):
    return _semanage.semanage_is_managed(arg1)

def semanage_connect(arg1):
    return _semanage.semanage_connect(arg1)

def semanage_disconnect(arg1):
    return _semanage.semanage_disconnect(arg1)

def semanage_begin_transaction(arg1):
    return _semanage.semanage_begin_transaction(arg1)

def semanage_commit(arg1):
    return _semanage.semanage_commit(arg1)
SEMANAGE_CAN_READ = _semanage.SEMANAGE_CAN_READ
SEMANAGE_CAN_WRITE = _semanage.SEMANAGE_CAN_WRITE

def semanage_access_check(sh):
    return _semanage.semanage_access_check(sh)

def semanage_is_connected(sh):
    return _semanage.semanage_is_connected(sh)

def semanage_mls_enabled(sh):
    return _semanage.semanage_mls_enabled(sh)

def semanage_set_root(path):
    return _semanage.semanage_set_root(path)

def semanage_root():
    return _semanage.semanage_root()

def semanage_get_preserve_tunables(handle):
    return _semanage.semanage_get_preserve_tunables(handle)

def semanage_set_preserve_tunables(handle, preserve_tunables):
    return _semanage.semanage_set_preserve_tunables(handle, preserve_tunables)

def semanage_get_ignore_module_cache(handle):
    return _semanage.semanage_get_ignore_module_cache(handle)

def semanage_set_ignore_module_cache(handle, ignore_module_cache):
    return _semanage.semanage_set_ignore_module_cache(handle, ignore_module_cache)

def semanage_set_store_root(sh, store_root):
    return _semanage.semanage_set_store_root(sh, store_root)

def semanage_module_install(arg1, module_data, data_len, name, ext_lang):
    return _semanage.semanage_module_install(arg1, module_data, data_len, name, ext_lang)

def semanage_module_install_file(arg1, module_name):
    return _semanage.semanage_module_install_file(arg1, module_name)

def semanage_module_remove(arg1, module_name):
    return _semanage.semanage_module_remove(arg1, module_name)

def semanage_module_extract(sh, modkey, extract_cil):
    return _semanage.semanage_module_extract(sh, modkey, extract_cil)

def semanage_module_list(arg1):
    return _semanage.semanage_module_list(arg1)

def semanage_module_info_datum_destroy(arg1):
    return _semanage.semanage_module_info_datum_destroy(arg1)

def semanage_module_list_nth(list, n):
    return _semanage.semanage_module_list_nth(list, n)

def semanage_module_get_name(arg1):
    return _semanage.semanage_module_get_name(arg1)

def semanage_module_info_create(sh):
    return _semanage.semanage_module_info_create(sh)

def semanage_module_info_destroy(handle, modinfo):
    return _semanage.semanage_module_info_destroy(handle, modinfo)

def semanage_module_info_get_priority(sh, modinfo):
    return _semanage.semanage_module_info_get_priority(sh, modinfo)

def semanage_module_info_get_name(sh, modinfo):
    return _semanage.semanage_module_info_get_name(sh, modinfo)

def semanage_module_info_get_lang_ext(sh, modinfo):
    return _semanage.semanage_module_info_get_lang_ext(sh, modinfo)

def semanage_module_info_get_enabled(sh, modinfo):
    return _semanage.semanage_module_info_get_enabled(sh, modinfo)

def semanage_module_info_set_priority(sh, modinfo, priority):
    return _semanage.semanage_module_info_set_priority(sh, modinfo, priority)

def semanage_module_info_set_name(sh, modinfo, name):
    return _semanage.semanage_module_info_set_name(sh, modinfo, name)

def semanage_module_info_set_lang_ext(sh, modinfo, lang_ext):
    return _semanage.semanage_module_info_set_lang_ext(sh, modinfo, lang_ext)

def semanage_module_info_set_enabled(sh, modinfo, enabled):
    return _semanage.semanage_module_info_set_enabled(sh, modinfo, enabled)

def semanage_module_key_create(sh):
    return _semanage.semanage_module_key_create(sh)

def semanage_module_key_destroy(sh, modkey):
    return _semanage.semanage_module_key_destroy(sh, modkey)

def semanage_module_key_get_name(sh, modkey):
    return _semanage.semanage_module_key_get_name(sh, modkey)

def semanage_module_key_get_priority(sh, modkey):
    return _semanage.semanage_module_key_get_priority(sh, modkey)

def semanage_module_key_set_name(sh, modkey, name):
    return _semanage.semanage_module_key_set_name(sh, modkey, name)

def semanage_module_key_set_priority(sh, modkey, priority):
    return _semanage.semanage_module_key_set_priority(sh, modkey, priority)

def semanage_module_set_enabled(sh, modkey, enabled):
    return _semanage.semanage_module_set_enabled(sh, modkey, enabled)

def semanage_module_get_module_info(sh, modkey):
    return _semanage.semanage_module_get_module_info(sh, modkey)

def semanage_module_list_all(sh):
    return _semanage.semanage_module_list_all(sh)

def semanage_module_install_info(sh, modinfo, data, data_len):
    return _semanage.semanage_module_install_info(sh, modinfo, data, data_len)

def semanage_module_remove_key(sh, modkey):
    return _semanage.semanage_module_remove_key(sh, modkey)

def semanage_module_get_enabled(sh, modkey):
    return _semanage.semanage_module_get_enabled(sh, modkey)

def semanage_module_compute_checksum(sh, modkey, cil):
    return _semanage.semanage_module_compute_checksum(sh, modkey, cil)

def semanage_context_get_user(con):
    return _semanage.semanage_context_get_user(con)

def semanage_context_set_user(handle, con, user):
    return _semanage.semanage_context_set_user(handle, con, user)

def semanage_context_get_role(con):
    return _semanage.semanage_context_get_role(con)

def semanage_context_set_role(handle, con, role):
    return _semanage.semanage_context_set_role(handle, con, role)

def semanage_context_get_type(con):
    return _semanage.semanage_context_get_type(con)

def semanage_context_set_type(handle, con, type):
    return _semanage.semanage_context_set_type(handle, con, type)

def semanage_context_get_mls(con):
    return _semanage.semanage_context_get_mls(con)

def semanage_context_set_mls(handle, con, mls_range):
    return _semanage.semanage_context_set_mls(handle, con, mls_range)

def semanage_context_create(handle):
    return _semanage.semanage_context_create(handle)

def semanage_context_clone(handle, con):
    return _semanage.semanage_context_clone(handle, con)

def semanage_context_free(con):
    return _semanage.semanage_context_free(con)

def semanage_context_from_string(handle, str):
    return _semanage.semanage_context_from_string(handle, str)

def semanage_context_to_string(handle, con):
    return _semanage.semanage_context_to_string(handle, con)

def semanage_bool_key_create(handle, name):
    return _semanage.semanage_bool_key_create(handle, name)

def semanage_bool_key_extract(handle, boolean):
    return _semanage.semanage_bool_key_extract(handle, boolean)

def semanage_bool_key_free(key):
    return _semanage.semanage_bool_key_free(key)

def semanage_bool_compare(boolean, key):
    return _semanage.semanage_bool_compare(boolean, key)

def semanage_bool_compare2(boolean, boolean2):
    return _semanage.semanage_bool_compare2(boolean, boolean2)

def semanage_bool_get_name(boolean):
    return _semanage.semanage_bool_get_name(boolean)

def semanage_bool_set_name(handle, boolean, name):
    return _semanage.semanage_bool_set_name(handle, boolean, name)

def semanage_bool_get_value(boolean):
    return _semanage.semanage_bool_get_value(boolean)

def semanage_bool_set_value(boolean, value):
    return _semanage.semanage_bool_set_value(boolean, value)

def semanage_bool_create(handle):
    return _semanage.semanage_bool_create(handle)

def semanage_bool_clone(handle, boolean):
    return _semanage.semanage_bool_clone(handle, boolean)

def semanage_bool_free(boolean):
    return _semanage.semanage_bool_free(boolean)

def semanage_bool_query(handle, key):
    return _semanage.semanage_bool_query(handle, key)

def semanage_bool_exists(handle, key):
    return _semanage.semanage_bool_exists(handle, key)

def semanage_bool_count(handle):
    return _semanage.semanage_bool_count(handle)

def semanage_bool_iterate(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate(handle, handler, handler_arg)

def semanage_bool_list(handle):
    return _semanage.semanage_bool_list(handle)

def semanage_bool_modify_local(handle, key, data):
    return _semanage.semanage_bool_modify_local(handle, key, data)

def semanage_bool_del_local(handle, key):
    return _semanage.semanage_bool_del_local(handle, key)

def semanage_bool_query_local(handle, key):
    return _semanage.semanage_bool_query_local(handle, key)

def semanage_bool_exists_local(handle, key):
    return _semanage.semanage_bool_exists_local(handle, key)

def semanage_bool_count_local(handle):
    return _semanage.semanage_bool_count_local(handle)

def semanage_bool_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate_local(handle, handler, handler_arg)

def semanage_bool_list_local(handle):
    return _semanage.semanage_bool_list_local(handle)

def semanage_bool_set_active(handle, key, data):
    return _semanage.semanage_bool_set_active(handle, key, data)

def semanage_bool_query_active(handle, key):
    return _semanage.semanage_bool_query_active(handle, key)

def semanage_bool_exists_active(handle, key):
    return _semanage.semanage_bool_exists_active(handle, key)

def semanage_bool_count_active(handle):
    return _semanage.semanage_bool_count_active(handle)

def semanage_bool_iterate_active(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate_active(handle, handler, handler_arg)

def semanage_bool_list_active(handle):
    return _semanage.semanage_bool_list_active(handle)

def semanage_iface_compare(iface, key):
    return _semanage.semanage_iface_compare(iface, key)

def semanage_iface_compare2(iface, iface2):
    return _semanage.semanage_iface_compare2(iface, iface2)

def semanage_iface_key_create(handle, name):
    return _semanage.semanage_iface_key_create(handle, name)

def semanage_iface_key_extract(handle, iface):
    return _semanage.semanage_iface_key_extract(handle, iface)

def semanage_iface_key_free(key):
    return _semanage.semanage_iface_key_free(key)

def semanage_iface_get_name(iface):
    return _semanage.semanage_iface_get_name(iface)

def semanage_iface_set_name(handle, iface, name):
    return _semanage.semanage_iface_set_name(handle, iface, name)

def semanage_iface_get_ifcon(iface):
    return _semanage.semanage_iface_get_ifcon(iface)

def semanage_iface_set_ifcon(handle, iface, con):
    return _semanage.semanage_iface_set_ifcon(handle, iface, con)

def semanage_iface_get_msgcon(iface):
    return _semanage.semanage_iface_get_msgcon(iface)

def semanage_iface_set_msgcon(handle, iface, con):
    return _semanage.semanage_iface_set_msgcon(handle, iface, con)

def semanage_iface_create(handle):
    return _semanage.semanage_iface_create(handle)

def semanage_iface_clone(handle, iface):
    return _semanage.semanage_iface_clone(handle, iface)

def semanage_iface_free(iface):
    return _semanage.semanage_iface_free(iface)

def semanage_iface_modify_local(handle, key, data):
    return _semanage.semanage_iface_modify_local(handle, key, data)

def semanage_iface_del_local(handle, key):
    return _semanage.semanage_iface_del_local(handle, key)

def semanage_iface_query_local(handle, key):
    return _semanage.semanage_iface_query_local(handle, key)

def semanage_iface_exists_local(handle, key):
    return _semanage.semanage_iface_exists_local(handle, key)

def semanage_iface_count_local(handle):
    return _semanage.semanage_iface_count_local(handle)

def semanage_iface_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_iface_iterate_local(handle, handler, handler_arg)

def semanage_iface_list_local(handle):
    return _semanage.semanage_iface_list_local(handle)

def semanage_iface_query(handle, key):
    return _semanage.semanage_iface_query(handle, key)

def semanage_iface_exists(handle, key):
    return _semanage.semanage_iface_exists(handle, key)

def semanage_iface_count(handle):
    return _semanage.semanage_iface_count(handle)

def semanage_iface_iterate(handle, handler, handler_arg):
    return _semanage.semanage_iface_iterate(handle, handler, handler_arg)

def semanage_iface_list(handle):
    return _semanage.semanage_iface_list(handle)

def semanage_user_key_create(handle, name):
    return _semanage.semanage_user_key_create(handle, name)

def semanage_user_key_extract(handle, user):
    return _semanage.semanage_user_key_extract(handle, user)

def semanage_user_key_free(key):
    return _semanage.semanage_user_key_free(key)

def semanage_user_compare(user, key):
    return _semanage.semanage_user_compare(user, key)

def semanage_user_compare2(user, user2):
    return _semanage.semanage_user_compare2(user, user2)

def semanage_user_get_name(user):
    return _semanage.semanage_user_get_name(user)

def semanage_user_set_name(handle, user, name):
    return _semanage.semanage_user_set_name(handle, user, name)

def semanage_user_get_prefix(user):
    return _semanage.semanage_user_get_prefix(user)

def semanage_user_set_prefix(handle, user, name):
    return _semanage.semanage_user_set_prefix(handle, user, name)

def semanage_user_get_mlslevel(user):
    return _semanage.semanage_user_get_mlslevel(user)

def semanage_user_set_mlslevel(handle, user, mls_level):
    return _semanage.semanage_user_set_mlslevel(handle, user, mls_level)

def semanage_user_get_mlsrange(user):
    return _semanage.semanage_user_get_mlsrange(user)

def semanage_user_set_mlsrange(handle, user, mls_range):
    return _semanage.semanage_user_set_mlsrange(handle, user, mls_range)

def semanage_user_get_num_roles(user):
    return _semanage.semanage_user_get_num_roles(user)

def semanage_user_add_role(handle, user, role):
    return _semanage.semanage_user_add_role(handle, user, role)

def semanage_user_del_role(user, role):
    return _semanage.semanage_user_del_role(user, role)

def semanage_user_has_role(user, role):
    return _semanage.semanage_user_has_role(user, role)

def semanage_user_get_roles(handle, user):
    return _semanage.semanage_user_get_roles(handle, user)

def semanage_user_set_roles(handle, user, num_roles):
    return _semanage.semanage_user_set_roles(handle, user, num_roles)

def semanage_user_create(handle):
    return _semanage.semanage_user_create(handle)

def semanage_user_clone(handle, user):
    return _semanage.semanage_user_clone(handle, user)

def semanage_user_free(user):
    return _semanage.semanage_user_free(user)

def semanage_user_modify_local(handle, key, data):
    return _semanage.semanage_user_modify_local(handle, key, data)

def semanage_user_del_local(handle, key):
    return _semanage.semanage_user_del_local(handle, key)

def semanage_user_query_local(handle, key):
    return _semanage.semanage_user_query_local(handle, key)

def semanage_user_exists_local(handle, key):
    return _semanage.semanage_user_exists_local(handle, key)

def semanage_user_count_local(handle):
    return _semanage.semanage_user_count_local(handle)

def semanage_user_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_user_iterate_local(handle, handler, handler_arg)

def semanage_user_list_local(handle):
    return _semanage.semanage_user_list_local(handle)

def semanage_user_query(handle, key):
    return _semanage.semanage_user_query(handle, key)

def semanage_user_exists(handle, key):
    return _semanage.semanage_user_exists(handle, key)

def semanage_user_count(handle):
    return _semanage.semanage_user_count(handle)

def semanage_user_iterate(handle, handler, handler_arg):
    return _semanage.semanage_user_iterate(handle, handler, handler_arg)

def semanage_user_list(handle):
    return _semanage.semanage_user_list(handle)
SEMANAGE_PROTO_UDP = _semanage.SEMANAGE_PROTO_UDP
SEMANAGE_PROTO_TCP = _semanage.SEMANAGE_PROTO_TCP
SEMANAGE_PROTO_DCCP = _semanage.SEMANAGE_PROTO_DCCP
SEMANAGE_PROTO_SCTP = _semanage.SEMANAGE_PROTO_SCTP

def semanage_port_compare(port, key):
    return _semanage.semanage_port_compare(port, key)

def semanage_port_compare2(port, port2):
    return _semanage.semanage_port_compare2(port, port2)

def semanage_port_key_create(handle, low, high, proto):
    return _semanage.semanage_port_key_create(handle, low, high, proto)

def semanage_port_key_extract(handle, port):
    return _semanage.semanage_port_key_extract(handle, port)

def semanage_port_key_free(key):
    return _semanage.semanage_port_key_free(key)

def semanage_port_get_proto(port):
    return _semanage.semanage_port_get_proto(port)

def semanage_port_set_proto(port, proto):
    return _semanage.semanage_port_set_proto(port, proto)

def semanage_port_get_proto_str(proto):
    return _semanage.semanage_port_get_proto_str(proto)

def semanage_port_get_low(port):
    return _semanage.semanage_port_get_low(port)

def semanage_port_get_high(port):
    return _semanage.semanage_port_get_high(port)

def semanage_port_set_port(port, port_num):
    return _semanage.semanage_port_set_port(port, port_num)

def semanage_port_set_range(port, low, high):
    return _semanage.semanage_port_set_range(port, low, high)

def semanage_port_get_con(port):
    return _semanage.semanage_port_get_con(port)

def semanage_port_set_con(handle, port, con):
    return _semanage.semanage_port_set_con(handle, port, con)

def semanage_port_create(handle):
    return _semanage.semanage_port_create(handle)

def semanage_port_clone(handle, port):
    return _semanage.semanage_port_clone(handle, port)

def semanage_port_free(port):
    return _semanage.semanage_port_free(port)

def semanage_port_modify_local(handle, key, data):
    return _semanage.semanage_port_modify_local(handle, key, data)

def semanage_port_del_local(handle, key):
    return _semanage.semanage_port_del_local(handle, key)

def semanage_port_query_local(handle, key):
    return _semanage.semanage_port_query_local(handle, key)

def semanage_port_exists_local(handle, key):
    return _semanage.semanage_port_exists_local(handle, key)

def semanage_port_count_local(handle):
    return _semanage.semanage_port_count_local(handle)

def semanage_port_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_port_iterate_local(handle, handler, handler_arg)

def semanage_port_list_local(handle):
    return _semanage.semanage_port_list_local(handle)

def semanage_port_query(handle, key):
    return _semanage.semanage_port_query(handle, key)

def semanage_port_exists(handle, key):
    return _semanage.semanage_port_exists(handle, key)

def semanage_port_count(handle):
    return _semanage.semanage_port_count(handle)

def semanage_port_iterate(handle, handler, handler_arg):
    return _semanage.semanage_port_iterate(handle, handler, handler_arg)

def semanage_port_list(handle):
    return _semanage.semanage_port_list(handle)

def semanage_ibpkey_compare(ibpkey, key):
    return _semanage.semanage_ibpkey_compare(ibpkey, key)

def semanage_ibpkey_compare2(ibpkey, ibpkey2):
    return _semanage.semanage_ibpkey_compare2(ibpkey, ibpkey2)

def semanage_ibpkey_key_create(handle, subnet_prefix, low, high):
    return _semanage.semanage_ibpkey_key_create(handle, subnet_prefix, low, high)

def semanage_ibpkey_key_extract(handle, ibpkey):
    return _semanage.semanage_ibpkey_key_extract(handle, ibpkey)

def semanage_ibpkey_key_free(key):
    return _semanage.semanage_ibpkey_key_free(key)

def semanage_ibpkey_get_subnet_prefix(handle, ibpkey):
    return _semanage.semanage_ibpkey_get_subnet_prefix(handle, ibpkey)

def semanage_ibpkey_get_subnet_prefix_bytes(ibpkey):
    return _semanage.semanage_ibpkey_get_subnet_prefix_bytes(ibpkey)

def semanage_ibpkey_set_subnet_prefix(handle, ibpkey, subnet_prefix):
    return _semanage.semanage_ibpkey_set_subnet_prefix(handle, ibpkey, subnet_prefix)

def semanage_ibpkey_set_subnet_prefix_bytes(ibpkey, subnet_prefix):
    return _semanage.semanage_ibpkey_set_subnet_prefix_bytes(ibpkey, subnet_prefix)

def semanage_ibpkey_get_low(ibpkey):
    return _semanage.semanage_ibpkey_get_low(ibpkey)

def semanage_ibpkey_get_high(ibpkey):
    return _semanage.semanage_ibpkey_get_high(ibpkey)

def semanage_ibpkey_set_pkey(ibpkey, pkey_num):
    return _semanage.semanage_ibpkey_set_pkey(ibpkey, pkey_num)

def semanage_ibpkey_set_range(ibpkey, low, high):
    return _semanage.semanage_ibpkey_set_range(ibpkey, low, high)

def semanage_ibpkey_get_con(ibpkey):
    return _semanage.semanage_ibpkey_get_con(ibpkey)

def semanage_ibpkey_set_con(handle, ibpkey, con):
    return _semanage.semanage_ibpkey_set_con(handle, ibpkey, con)

def semanage_ibpkey_create(handle):
    return _semanage.semanage_ibpkey_create(handle)

def semanage_ibpkey_clone(handle, ibpkey):
    return _semanage.semanage_ibpkey_clone(handle, ibpkey)

def semanage_ibpkey_free(ibpkey):
    return _semanage.semanage_ibpkey_free(ibpkey)

def semanage_ibpkey_modify_local(handle, key, data):
    return _semanage.semanage_ibpkey_modify_local(handle, key, data)

def semanage_ibpkey_del_local(handle, key):
    return _semanage.semanage_ibpkey_del_local(handle, key)

def semanage_ibpkey_query_local(handle, key):
    return _semanage.semanage_ibpkey_query_local(handle, key)

def semanage_ibpkey_exists_local(handle, key):
    return _semanage.semanage_ibpkey_exists_local(handle, key)

def semanage_ibpkey_count_local(handle):
    return _semanage.semanage_ibpkey_count_local(handle)

def semanage_ibpkey_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_ibpkey_iterate_local(handle, handler, handler_arg)

def semanage_ibpkey_list_local(handle):
    return _semanage.semanage_ibpkey_list_local(handle)

def semanage_ibpkey_query(handle, key):
    return _semanage.semanage_ibpkey_query(handle, key)

def semanage_ibpkey_exists(handle, key):
    return _semanage.semanage_ibpkey_exists(handle, key)

def semanage_ibpkey_count(handle):
    return _semanage.semanage_ibpkey_count(handle)

def semanage_ibpkey_iterate(handle, handler, handler_arg):
    return _semanage.semanage_ibpkey_iterate(handle, handler, handler_arg)

def semanage_ibpkey_list(handle):
    return _semanage.semanage_ibpkey_list(handle)

def semanage_ibendport_compare(ibendport, key):
    return _semanage.semanage_ibendport_compare(ibendport, key)

def semanage_ibendport_compare2(ibendport, ibendport2):
    return _semanage.semanage_ibendport_compare2(ibendport, ibendport2)

def semanage_ibendport_key_create(handle, ibdev_name, port):
    return _semanage.semanage_ibendport_key_create(handle, ibdev_name, port)

def semanage_ibendport_key_extract(handle, ibendport):
    return _semanage.semanage_ibendport_key_extract(handle, ibendport)

def semanage_ibendport_key_free(key):
    return _semanage.semanage_ibendport_key_free(key)

def semanage_ibendport_get_ibdev_name(handle, ibendport):
    return _semanage.semanage_ibendport_get_ibdev_name(handle, ibendport)

def semanage_ibendport_set_ibdev_name(handle, ibendport, ibdev_name):
    return _semanage.semanage_ibendport_set_ibdev_name(handle, ibendport, ibdev_name)

def semanage_ibendport_get_port(ibendport):
    return _semanage.semanage_ibendport_get_port(ibendport)

def semanage_ibendport_set_port(ibendport, port):
    return _semanage.semanage_ibendport_set_port(ibendport, port)

def semanage_ibendport_get_con(ibendport):
    return _semanage.semanage_ibendport_get_con(ibendport)

def semanage_ibendport_set_con(handle, ibendport, con):
    return _semanage.semanage_ibendport_set_con(handle, ibendport, con)

def semanage_ibendport_create(handle):
    return _semanage.semanage_ibendport_create(handle)

def semanage_ibendport_clone(handle, ibendport):
    return _semanage.semanage_ibendport_clone(handle, ibendport)

def semanage_ibendport_free(ibendport):
    return _semanage.semanage_ibendport_free(ibendport)

def semanage_ibendport_modify_local(handle, key, data):
    return _semanage.semanage_ibendport_modify_local(handle, key, data)

def semanage_ibendport_del_local(handle, key):
    return _semanage.semanage_ibendport_del_local(handle, key)

def semanage_ibendport_query_local(handle, key):
    return _semanage.semanage_ibendport_query_local(handle, key)

def semanage_ibendport_exists_local(handle, key):
    return _semanage.semanage_ibendport_exists_local(handle, key)

def semanage_ibendport_count_local(handle):
    return _semanage.semanage_ibendport_count_local(handle)

def semanage_ibendport_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_ibendport_iterate_local(handle, handler, handler_arg)

def semanage_ibendport_list_local(handle):
    return _semanage.semanage_ibendport_list_local(handle)

def semanage_ibendport_query(handle, key):
    return _semanage.semanage_ibendport_query(handle, key)

def semanage_ibendport_exists(handle, key):
    return _semanage.semanage_ibendport_exists(handle, key)

def semanage_ibendport_count(handle):
    return _semanage.semanage_ibendport_count(handle)

def semanage_ibendport_iterate(handle, handler, handler_arg):
    return _semanage.semanage_ibendport_iterate(handle, handler, handler_arg)

def semanage_ibendport_list(handle):
    return _semanage.semanage_ibendport_list(handle)

def semanage_fcontext_compare(fcontext, key):
    return _semanage.semanage_fcontext_compare(fcontext, key)

def semanage_fcontext_compare2(fcontext, fcontext2):
    return _semanage.semanage_fcontext_compare2(fcontext, fcontext2)

def semanage_fcontext_key_create(handle, expr, type):
    return _semanage.semanage_fcontext_key_create(handle, expr, type)

def semanage_fcontext_key_extract(handle, fcontext):
    return _semanage.semanage_fcontext_key_extract(handle, fcontext)

def semanage_fcontext_key_free(key):
    return _semanage.semanage_fcontext_key_free(key)

def semanage_fcontext_get_expr(fcontext):
    return _semanage.semanage_fcontext_get_expr(fcontext)

def semanage_fcontext_set_expr(handle, fcontext, expr):
    return _semanage.semanage_fcontext_set_expr(handle, fcontext, expr)
SEMANAGE_FCONTEXT_ALL = _semanage.SEMANAGE_FCONTEXT_ALL
SEMANAGE_FCONTEXT_REG = _semanage.SEMANAGE_FCONTEXT_REG
SEMANAGE_FCONTEXT_DIR = _semanage.SEMANAGE_FCONTEXT_DIR
SEMANAGE_FCONTEXT_CHAR = _semanage.SEMANAGE_FCONTEXT_CHAR
SEMANAGE_FCONTEXT_BLOCK = _semanage.SEMANAGE_FCONTEXT_BLOCK
SEMANAGE_FCONTEXT_SOCK = _semanage.SEMANAGE_FCONTEXT_SOCK
SEMANAGE_FCONTEXT_LINK = _semanage.SEMANAGE_FCONTEXT_LINK
SEMANAGE_FCONTEXT_PIPE = _semanage.SEMANAGE_FCONTEXT_PIPE

def semanage_fcontext_get_type(fcontext):
    return _semanage.semanage_fcontext_get_type(fcontext)

def semanage_fcontext_get_type_str(type):
    return _semanage.semanage_fcontext_get_type_str(type)

def semanage_fcontext_set_type(fcontext, type):
    return _semanage.semanage_fcontext_set_type(fcontext, type)

def semanage_fcontext_get_con(fcontext):
    return _semanage.semanage_fcontext_get_con(fcontext)

def semanage_fcontext_set_con(handle, fcontext, con):
    return _semanage.semanage_fcontext_set_con(handle, fcontext, con)

def semanage_fcontext_create(handle):
    return _semanage.semanage_fcontext_create(handle)

def semanage_fcontext_clone(handle, fcontext):
    return _semanage.semanage_fcontext_clone(handle, fcontext)

def semanage_fcontext_free(fcontext):
    return _semanage.semanage_fcontext_free(fcontext)

def semanage_fcontext_modify_local(handle, key, data):
    return _semanage.semanage_fcontext_modify_local(handle, key, data)

def semanage_fcontext_del_local(handle, key):
    return _semanage.semanage_fcontext_del_local(handle, key)

def semanage_fcontext_query_local(handle, key):
    return _semanage.semanage_fcontext_query_local(handle, key)

def semanage_fcontext_exists_local(handle, key):
    return _semanage.semanage_fcontext_exists_local(handle, key)

def semanage_fcontext_count_local(handle):
    return _semanage.semanage_fcontext_count_local(handle)

def semanage_fcontext_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_fcontext_iterate_local(handle, handler, handler_arg)

def semanage_fcontext_list_local(handle):
    return _semanage.semanage_fcontext_list_local(handle)

def semanage_fcontext_query(handle, key):
    return _semanage.semanage_fcontext_query(handle, key)

def semanage_fcontext_exists(handle, key):
    return _semanage.semanage_fcontext_exists(handle, key)

def semanage_fcontext_count(handle):
    return _semanage.semanage_fcontext_count(handle)

def semanage_fcontext_iterate(handle, handler, handler_arg):
    return _semanage.semanage_fcontext_iterate(handle, handler, handler_arg)

def semanage_fcontext_list(handle):
    return _semanage.semanage_fcontext_list(handle)

def semanage_fcontext_list_homedirs(handle):
    return _semanage.semanage_fcontext_list_homedirs(handle)

def semanage_seuser_key_create(handle, name):
    return _semanage.semanage_seuser_key_create(handle, name)

def semanage_seuser_key_extract(handle, seuser):
    return _semanage.semanage_seuser_key_extract(handle, seuser)

def semanage_seuser_key_free(key):
    return _semanage.semanage_seuser_key_free(key)

def semanage_seuser_compare(seuser, key):
    return _semanage.semanage_seuser_compare(seuser, key)

def semanage_seuser_compare2(seuser, seuser2):
    return _semanage.semanage_seuser_compare2(seuser, seuser2)

def semanage_seuser_get_name(seuser):
    return _semanage.semanage_seuser_get_name(seuser)

def semanage_seuser_set_name(handle, seuser, name):
    return _semanage.semanage_seuser_set_name(handle, seuser, name)

def semanage_seuser_get_sename(seuser):
    return _semanage.semanage_seuser_get_sename(seuser)

def semanage_seuser_set_sename(handle, seuser, sename):
    return _semanage.semanage_seuser_set_sename(handle, seuser, sename)

def semanage_seuser_get_mlsrange(seuser):
    return _semanage.semanage_seuser_get_mlsrange(seuser)

def semanage_seuser_set_mlsrange(handle, seuser, mls_range):
    return _semanage.semanage_seuser_set_mlsrange(handle, seuser, mls_range)

def semanage_seuser_create(handle):
    return _semanage.semanage_seuser_create(handle)

def semanage_seuser_clone(handle, seuser):
    return _semanage.semanage_seuser_clone(handle, seuser)

def semanage_seuser_free(seuser):
    return _semanage.semanage_seuser_free(seuser)

def semanage_seuser_modify_local(handle, key, data):
    return _semanage.semanage_seuser_modify_local(handle, key, data)

def semanage_seuser_del_local(handle, key):
    return _semanage.semanage_seuser_del_local(handle, key)

def semanage_seuser_query_local(handle, key):
    return _semanage.semanage_seuser_query_local(handle, key)

def semanage_seuser_exists_local(handle, key):
    return _semanage.semanage_seuser_exists_local(handle, key)

def semanage_seuser_count_local(handle):
    return _semanage.semanage_seuser_count_local(handle)

def semanage_seuser_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_seuser_iterate_local(handle, handler, handler_arg)

def semanage_seuser_list_local(handle):
    return _semanage.semanage_seuser_list_local(handle)

def semanage_seuser_query(handle, key):
    return _semanage.semanage_seuser_query(handle, key)

def semanage_seuser_exists(handle, key):
    return _semanage.semanage_seuser_exists(handle, key)

def semanage_seuser_count(handle):
    return _semanage.semanage_seuser_count(handle)

def semanage_seuser_iterate(handle, handler, handler_arg):
    return _semanage.semanage_seuser_iterate(handle, handler, handler_arg)

def semanage_seuser_list(handle):
    return _semanage.semanage_seuser_list(handle)
SEMANAGE_PROTO_IP4 = _semanage.SEMANAGE_PROTO_IP4
SEMANAGE_PROTO_IP6 = _semanage.SEMANAGE_PROTO_IP6

def semanage_node_compare(node, key):
    return _semanage.semanage_node_compare(node, key)

def semanage_node_compare2(node, node2):
    return _semanage.semanage_node_compare2(node, node2)

def semanage_node_key_create(handle, addr, mask, proto):
    return _semanage.semanage_node_key_create(handle, addr, mask, proto)

def semanage_node_key_extract(handle, node):
    return _semanage.semanage_node_key_extract(handle, node)

def semanage_node_key_free(key):
    return _semanage.semanage_node_key_free(key)

def semanage_node_get_addr(handle, node):
    return _semanage.semanage_node_get_addr(handle, node)

def semanage_node_get_addr_bytes(handle, node):
    return _semanage.semanage_node_get_addr_bytes(handle, node)

def semanage_node_set_addr(handle, node, proto, addr):
    return _semanage.semanage_node_set_addr(handle, node, proto, addr)

def semanage_node_set_addr_bytes(handle, node, addr, addr_sz):
    return _semanage.semanage_node_set_addr_bytes(handle, node, addr, addr_sz)

def semanage_node_get_mask(handle, node):
    return _semanage.semanage_node_get_mask(handle, node)

def semanage_node_get_mask_bytes(handle, node):
    return _semanage.semanage_node_get_mask_bytes(handle, node)

def semanage_node_set_mask(handle, node, proto, mask):
    return _semanage.semanage_node_set_mask(handle, node, proto, mask)

def semanage_node_set_mask_bytes(handle, node, mask, mask_sz):
    return _semanage.semanage_node_set_mask_bytes(handle, node, mask, mask_sz)

def semanage_node_get_proto(node):
    return _semanage.semanage_node_get_proto(node)

def semanage_node_set_proto(node, proto):
    return _semanage.semanage_node_set_proto(node, proto)

def semanage_node_get_proto_str(proto):
    return _semanage.semanage_node_get_proto_str(proto)

def semanage_node_get_con(node):
    return _semanage.semanage_node_get_con(node)

def semanage_node_set_con(handle, node, con):
    return _semanage.semanage_node_set_con(handle, node, con)

def semanage_node_create(handle):
    return _semanage.semanage_node_create(handle)

def semanage_node_clone(handle, node):
    return _semanage.semanage_node_clone(handle, node)

def semanage_node_free(node):
    return _semanage.semanage_node_free(node)

def semanage_node_modify_local(handle, key, data):
    return _semanage.semanage_node_modify_local(handle, key, data)

def semanage_node_del_local(handle, key):
    return _semanage.semanage_node_del_local(handle, key)

def semanage_node_query_local(handle, key):
    return _semanage.semanage_node_query_local(handle, key)

def semanage_node_exists_local(handle, key):
    return _semanage.semanage_node_exists_local(handle, key)

def semanage_node_count_local(handle):
    return _semanage.semanage_node_count_local(handle)

def semanage_node_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_node_iterate_local(handle, handler, handler_arg)

def semanage_node_list_local(handle):
    return _semanage.semanage_node_list_local(handle)

def semanage_node_query(handle, key):
    return _semanage.semanage_node_query(handle, key)

def semanage_node_exists(handle, key):
    return _semanage.semanage_node_exists(handle, key)

def semanage_node_count(handle):
    return _semanage.semanage_node_count(handle)

def semanage_node_iterate(handle, handler, handler_arg):
    return _semanage.semanage_node_iterate(handle, handler, handler_arg)

def semanage_node_list(handle):
    return _semanage.semanage_node_list(handle)





# Copyright 2016, Tresys Technology, LLC
# Copyright 2018, Chris PeBenito <pebenito@ieee.org>
#
# SPDX-License-Identifier: LGPL-2.1-only
#
from collections import defaultdict
from typing import NamedTuple, Set, Optional, Union

from ..exception import MLSDisabled
from ..policyrep import Level, Range, Role, User

from .descriptors import DiffResultDescriptor
from .difference import Difference, SymbolWrapper
from .mls import LevelWrapper, RangeWrapper
from .roles import role_wrapper_factory
from .typing import SymbolCache

_users_cache: SymbolCache[User] = defaultdict(dict)


class ModifiedUser(NamedTuple):

    """Difference details for a modified user."""

    added_roles: Set[Role]
    removed_roles: Set[Role]
    matched_roles: Set[Role]
    added_level: Optional[Union[Level, str]]
    removed_level: Optional[Union[Level, str]]
    added_range: Optional[Union[Range, str]]
    removed_range: Optional[Union[Range, str]]


def user_wrapper_factory(user: User) -> SymbolWrapper[User]:
    """
    Wrap users from the specified policy.

    This caches results to prevent duplicate wrapper
    objects in memory.
    """
    try:
        return _users_cache[user.policy][user]
    except KeyError:
        r = SymbolWrapper(user)
        _users_cache[user.policy][user] = r
        return r


class UsersDifference(Difference):

    """Determine the difference in users between two policies."""

    added_users = DiffResultDescriptor("diff_users")
    removed_users = DiffResultDescriptor("diff_users")
    modified_users = DiffResultDescriptor("diff_users")

    def diff_users(self) -> None:
        """Generate the difference in users between the policies."""

        self.log.info(
            "Generating user differences from {0.left_policy} to {0.right_policy}".format(self))

        self.added_users, self.removed_users, matched_users = self._set_diff(
            (user_wrapper_factory(r) for r in self.left_policy.users()),
            (user_wrapper_factory(r) for r in self.right_policy.users()))

        self.modified_users = dict()

        for left_user, right_user in matched_users:
            # Criteria for modified users
            # 1. change to role set, or
            # 2. change to default level, or
            # 3. change to range
            added_roles, removed_roles, matched_roles = self._set_diff(
                (role_wrapper_factory(r) for r in left_user.roles),
                (role_wrapper_factory(r) for r in right_user.roles))

            # keep wrapped and unwrapped MLS objects here so there
            # are not several nested try blocks
            left_level_wrap: Optional[LevelWrapper]
            left_range_wrap: Optional[RangeWrapper]
            left_level: Union[Level, str]
            left_range: Union[Range, str]
            right_level_wrap: Optional[LevelWrapper]
            right_range_wrap: Optional[RangeWrapper]
            right_level: Union[Level, str]
            right_range: Union[Range, str]
            added_level: Optional[Union[Level, str]]
            added_range: Optional[Union[Range, str]]
            removed_level: Optional[Union[Level, str]]
            removed_range: Optional[Union[Range, str]]
            try:
                left_level_wrap = LevelWrapper(left_user.mls_level)
                left_range_wrap = RangeWrapper(left_user.mls_range)
                left_level = left_user.mls_level
                left_range = left_user.mls_range
            except MLSDisabled:
                left_level_wrap = None
                left_range_wrap = None
                left_level = "None (MLS Disabled)"
                left_range = "None (MLS Disabled)"

            try:
                right_level_wrap = LevelWrapper(right_user.mls_level)
                right_range_wrap = RangeWrapper(right_user.mls_range)
                right_level = right_user.mls_level
                right_range = right_user.mls_range
            except MLSDisabled:
                right_level_wrap = None
                right_range_wrap = None
                right_level = "None (MLS Disabled)"
                right_range = "None (MLS Disabled)"

            if left_level_wrap != right_level_wrap:
                added_level = right_level
                removed_level = left_level
            else:
                added_level = None
                removed_level = None

            if left_range_wrap != right_range_wrap:
                added_range = right_range
                removed_range = left_range
            else:
                added_range = None
                removed_range = None

            if added_roles or removed_roles or removed_level or removed_range:
                self.modified_users[left_user] = ModifiedUser(added_roles,
                                                              removed_roles,
                                                              matched_roles,
                                                              added_level,
                                                              removed_level,
                                                              added_range,
                                                              removed_range)

    #
    # Internal functions
    #
    def _reset_diff(self) -> None:
        """Reset diff results on policy changes."""
        self.log.debug("Resetting user differences")
        self.added_users = None
        self.removed_users = None
        self.modified_users = None



# Copyright 2015, Tresys Technology, LLC
# Copyright 2019, Chris PeBenito <pebenito@ieee.org>
#
# SPDX-License-Identifier: LGPL-2.1-only
#
# pylint: disable=attribute-defined-outside-init,no-member
import re
from logging import Logger
from typing import Iterable

from .descriptors import CriteriaDescriptor, CriteriaSetDescriptor, CriteriaPermissionSetDescriptor
from .policyrep import Context
from .util import match_in_set, match_regex, match_range, match_regex_or_set


class MatchAlias:

    """Mixin for matching an object's aliases."""

    alias = CriteriaDescriptor("alias_regex")
    alias_regex: bool = False

    def _match_alias_debug(self, log: Logger) -> None:
        """Emit log debugging info for alias matching."""
        log.debug("Alias: {0.alias}, regex: {0.alias_regex}".format(self))

    def _match_alias(self, obj):
        """
        Match the alias criteria

        Parameter:
        obj     An object with an alias generator method named "aliases"
        """

        if not self.alias:
            # if there is no criteria, everything matches.
            return True

        return match_in_set(obj.aliases(), self.alias, self.alias_regex)


class MatchContext:

    """
    Mixin for matching contexts.

    Class attributes:
    user            The user to match in the context.
    user_regex      If true, regular expression matching
                    will be used on the user.
    role            The role to match in the context.
    role_regex      If true, regular expression matching
                    will be used on the role.
    type_           The type to match in the context.
    type_regex      If true, regular expression matching
                    will be used on the type.
    range_          The range to match in the context.
    range_subset    If true, the criteria will match if it
                    is a subset of the context's range.
    range_overlap   If true, the criteria will match if it
                    overlaps any of the context's range.
    range_superset  If true, the criteria will match if it
                    is a superset of the context's range.
    range_proper    If true, use proper superset/subset
                    on range matching operations.
                    No effect if not using set operations.
    """

    user = CriteriaDescriptor("user_regex", "lookup_user")
    user_regex: bool = False
    role = CriteriaDescriptor("role_regex", "lookup_role")
    role_regex: bool = False
    type_ = CriteriaDescriptor("type_regex", "lookup_type")
    type_regex: bool = False
    range_ = CriteriaDescriptor(lookup_function="lookup_range")
    range_overlap: bool = False
    range_subset: bool = False
    range_superset: bool = False
    range_proper: bool = False

    def _match_context_debug(self, log: Logger):
        """Emit log debugging info for context matching."""
        log.debug("User: {0.user!r}, regex: {0.user_regex}".format(self))
        log.debug("Role: {0.role!r}, regex: {0.role_regex}".format(self))
        log.debug("Type: {0.type_!r}, regex: {0.type_regex}".format(self))
        log.debug("Range: {0.range_!r}, subset: {0.range_subset}, overlap: {0.range_overlap}, "
                  "superset: {0.range_superset}, proper: {0.range_proper}".format(self))

    def _match_context(self, context: Context) -> bool:
        """
        Match the context criteria.

        Parameter:
        obj     An object with context attributes "user", "role",
                "type_" and "range_".
        """

        if self.user and not match_regex(
                context.user,
                self.user,
                self.user_regex):
            return False

        if self.role and not match_regex(
                context.role,
                self.role,
                self.role_regex):
            return False

        if self.type_ and not match_regex(
                context.type_,
                self.type_,
                self.type_regex):
            return False

        if self.range_ and not match_range(
                context.range_,
                self.range_,
                self.range_subset,
                self.range_overlap,
                self.range_superset,
                self.range_proper):
            return False

        return True


class MatchName:

    """Mixin for matching an object's name with alias dereferencing."""

    name = CriteriaDescriptor("name_regex")
    name_regex: bool = False
    alias_deref: bool = False

    def _match_name_debug(self, log: Logger) -> None:
        """Log debugging messages for name matching."""
        log.debug("Name: {0.name!r}, regex: {0.name_regex}, deref: {0.alias_deref}".format(self))

    def _match_name(self, obj):
        """Match the object to the name criteria."""
        if not self.name:
            # if there is no criteria, everything matches.
            return True

        if self.alias_deref:
            return match_regex(obj, self.name, self.name_regex) or \
                match_in_set(obj.aliases(), self.name, self.name_regex)
        else:
            return match_regex(obj, self.name, self.name_regex)


class MatchObjClass:

    """Mixin for matching an object's class."""

    tclass = CriteriaSetDescriptor("tclass_regex", "lookup_class")
    tclass_regex: bool = False

    def _match_object_class_debug(self, log: Logger) -> None:
        """Emit log debugging info for permission matching."""
        log.debug("Class: {0.tclass!r}, regex: {0.tclass_regex}".format(self))

    def _match_object_class(self, obj):
        """
        Match the object class criteria

        Parameter:
        obj     An object with an object class attribute named "tclass"
        """

        if not self.tclass:
            # if there is no criteria, everything matches.
            return True
        elif self.tclass_regex:
            return bool(self.tclass.search(str(obj.tclass)))
        else:
            return obj.tclass in self.tclass


class MatchPermission:

    """Mixin for matching an object's permissions."""

    perms = CriteriaPermissionSetDescriptor(name_regex="perms_regex")
    perms_equal: bool = False
    perms_regex: bool = False
    perms_subset: bool = False

    def _match_perms_debug(self, log: Logger):
        """Emit log debugging info for permission matching."""
        log.debug("Perms: {0.perms!r}, regex: {0.perms_regex}, eq: {0.perms_equal}, "
                  "subset: {0.perms_subset!r}".format(self))

    def _match_perms(self, obj):
        """
        Match the permission criteria

        Parameter:
        obj     An object with a permission set class attribute named "perms"
        """

        if not self.perms:
            # if there is no criteria, everything matches.
            return True

        if self.perms_subset:
            return obj.perms >= self.perms
        else:
            return match_regex_or_set(obj.perms, self.perms, self.perms_equal, self.perms_regex)



# Copyright 2020, Microsoft Corporation
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from . import assertrbac
from . import assertte
from . import emptyattr
from . import roexec

from .checker import PolicyChecker



# -*- coding: utf-8 -*-

# Copyright (C) 2016-2018 g10 Code GmbH
# Copyright (C) 2015 Ben McGinnes <ben@adversary.org>
# Copyright (C) 2004 Igor Belyi <belyi@users.sourceforge.net>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

from __future__ import absolute_import, print_function

from . import gpgme

del absolute_import, print_function

productname = 'gpg'
versionstr = "1.15.1"
gpgme_versionstr = gpgme.GPGME_VERSION
in_tree_build = bool(gpgme.cvar.gpg_in_tree_build)
is_beta = False

versionlist = versionstr.split(".")
major = versionlist[0]
minor = versionlist[1]
patch = versionlist[2]

versionintlist = []
versionintlist.append(int(major))
versionintlist.append(int(minor))

try:
    int(patch)
except ValueError as e:
    is_beta = True

if is_beta is False:
    versionintlist.append(int(patch))
else:
    try:
        beta_patch = patch.split("-")
        versionintlist.append(int(beta_patch[0]))
    except Exception as e:
        # This should never happen, if it does then lodge a bug report.
        versionintlist.append(-1)

copyright = """\
Copyright (C) 2016-2018 g10 Code GmbH
Copyright (C) 2015 Benjamin D. McGinnes
Copyright (C) 2014-2015 Martin Albrecht
Copyright (C) 2004-2008 Igor Belyi
Copyright (C) 2002 John Goerzen"""

author = "The GnuPG hackers"
author_email = "gnupg-devel@gnupg.org"

description = "Python support for GPGME GnuPG cryptography library"
homepage = "https://gnupg.org"

license = """Copyright (C) 2016-2018 g10 Code GmbH
Copyright (C) 2015 Benjamin D. McGinnes <ben@adversary.org>
Copyright (C) 2014, 2015 Martin Albrecht <martinralbrecht@googlemail.com>
Copyright (C) 2004, 2008 Igor Belyi <belyi@users.sourceforge.net>
Copyright (C) 2002 John Goerzen <jgoerzen@complete.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA"""

# Interface hygiene.  Keep this at the end.
del gpgme



# Copyright 2020, Microsoft Corporation
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import logging
from collections import defaultdict
from typing import Dict, List, Set

from ..policyrep import AnyTERule, Type
from ..terulequery import TERuleQuery

from .checkermodule import CheckerModule
from .descriptors import ConfigSetDescriptor


EXEMPT_WRITE = "exempt_write_domain"
EXEMPT_EXEC = "exempt_exec_domain"
EXEMPT_FILE = "exempt_file"


class ReadOnlyExecutables(CheckerModule):

    """Checker module for asserting all executable files are read-only."""

    check_type = "ro_execs"
    check_config = frozenset((EXEMPT_WRITE, EXEMPT_EXEC, EXEMPT_FILE))

    exempt_write_domain = ConfigSetDescriptor("lookup_type_or_attr", strict=False, expand=True)
    exempt_file = ConfigSetDescriptor("lookup_type_or_attr", strict=False, expand=True)
    exempt_exec_domain = ConfigSetDescriptor("lookup_type_or_attr", strict=False, expand=True)

    def __init__(self, policy, checkname, config) -> None:
        super().__init__(policy, checkname, config)
        self.log = logging.getLogger(__name__)

        self.exempt_write_domain = config.get(EXEMPT_WRITE)
        self.exempt_file = config.get(EXEMPT_FILE)
        self.exempt_exec_domain = config.get(EXEMPT_EXEC)

    def _collect_executables(self) -> Dict[Type, Set[AnyTERule]]:
        self.log.debug("Collecting list of executable file types.")
        self.log.debug("Ignore exec domains: {!r}".format(self.exempt_exec_domain))
        query = TERuleQuery(self.policy,
                            ruletype=("allow",),
                            tclass=("file",),
                            perms=("execute", "execute_no_trans"))

        collected = defaultdict(set)
        for rule in query.results():
            sources = set(rule.source.expand()) - self.exempt_exec_domain
            targets = set(rule.target.expand()) - self.exempt_file

            # ignore rule if source or target is an empty attr
            if not sources or not targets:
                self.log.debug("Ignoring execute rule: {}".format(rule))
                continue

            for t in targets:
                self.log.debug("Determined {} is executable by: {}".format(t, rule))
                collected[t].add(rule)

        return collected

    def run(self) -> List:
        self.log.info("Checking executables are read-only.")

        query = TERuleQuery(self.policy,
                            ruletype=("allow",),
                            tclass=("file",),
                            perms=("write", "append"))
        executables = self._collect_executables()
        failures = defaultdict(set)

        for exec_type in executables.keys():
            self.log.debug("Checking if executable type {} is writable.".format(exec_type))

            query.target = exec_type
            for rule in sorted(query.results()):
                if set(rule.source.expand()) - self.exempt_write_domain:
                    failures[exec_type].add(rule)

        for exec_type in sorted(failures.keys()):
            self.output.write("\n------------\n\n")
            self.output.write("Executable type {} is writable.\n\n".format(exec_type))
            self.output.write("Execute rules:\n")
            for rule in sorted(executables[exec_type]):
                self.output.write("    * {}\n".format(rule))

            self.output.write("\nWrite rules:\n")
            for rule in sorted(failures[exec_type]):
                self.log_fail(str(rule))

        self.log.debug("{} failure(s)".format(len(failures)))
        return sorted(failures.keys())



# Copyright (C) 2004 Igor Belyi <belyi@users.sourceforge.net>
# Copyright (C) 2002 John Goerzen <jgoerzen@complete.org>
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

from __future__ import absolute_import, print_function, unicode_literals

from getpass import getpass

del absolute_import, print_function, unicode_literals


def passphrase_stdin(hint, desc, prev_bad, hook=None):
    """This is a sample callback that will read a passphrase from
    the terminal.  The hook here, if present, will be used to describe
    why the passphrase is needed."""
    why = ''
    if hook is not None:
        why = ' ' + hook
    if prev_bad:
        why += ' (again)'
    print("Please supply %s' password%s:" % (hint, why))
    return getpass()


def progress_stdout(what, type, current, total, hook=None):
    print("PROGRESS UPDATE: what = %s, type = %d, current = %d, total = %d" %
          (what, type, current, total))


def readcb_fh(count, hook):
    """A callback for data.  hook should be a Python file-like object."""
    if count:
        # Should return '' on EOF
        return hook.read(count)
    else:
        # Wants to rewind.
        if not hasattr(hook, 'seek'):
            return None
        hook.seek(0, 0)
        return None



# Copyright 2014-2015, Tresys Technology, LLC
#
# SPDX-License-Identifier: LGPL-2.1-only
#
import logging
from typing import Iterable

from .mixins import MatchContext, MatchName
from .policyrep import Netifcon
from .query import PolicyQuery
from .util import match_regex


class NetifconQuery(MatchContext, MatchName, PolicyQuery):

    """
    Network interface context query.

    Parameter:
    policy          The policy to query.

    Keyword Parameters/Class attributes:
    name            The name of the network interface to match.
    name_regex      If true, regular expression matching will
                    be used for matching the name.
    user            The criteria to match the context's user.
    user_regex      If true, regular expression matching
                    will be used on the user.
    role            The criteria to match the context's role.
    role_regex      If true, regular expression matching
                    will be used on the role.
    type_           The criteria to match the context's type.
    type_regex      If true, regular expression matching
                    will be used on the type.
    range_          The criteria to match the context's range.
    range_subset    If true, the criteria will match if it is a subset
                    of the context's range.
    range_overlap   If true, the criteria will match if it overlaps
                    any of the context's range.
    range_superset  If true, the criteria will match if it is a superset
                    of the context's range.
    range_proper    If true, use proper superset/subset operations.
                    No effect if not using set operations.
    """

    def __init__(self, policy, **kwargs) -> None:
        super(NetifconQuery, self).__init__(policy, **kwargs)
        self.log = logging.getLogger(__name__)

    def results(self) -> Iterable[Netifcon]:
        """Generator which yields all matching netifcons."""
        self.log.info("Generating netifcon results from {0.policy}".format(self))
        self._match_name_debug(self.log)
        self._match_context_debug(self.log)

        for netif in self.policy.netifcons():
            if self.name and not match_regex(
                    netif.netif,
                    self.name,
                    self.name_regex):
                continue

            if not self._match_context(netif.context):
                continue

            yield netif



# Copyright 2020, Microsoft Corporation
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import logging
from typing import List

from ..exception import InvalidType, InvalidCheckValue
from .checkermodule import CheckerModule
from .util import config_bool_value


ATTR_OPT = "attr"
MISSINOK_OPT = "missing_ok"


class EmptyTypeAttr(CheckerModule):

    """Checker module for asserting a type attribute is empty."""

    check_type = "empty_typeattr"
    check_config = frozenset((ATTR_OPT, MISSINOK_OPT))

    def __init__(self, policy, checkname, config) -> None:
        super().__init__(policy, checkname, config)
        self.log = logging.getLogger(__name__)
        self._attr = None
        self._missing_ok = False

        # this will make the check pass automatically
        # since the attribute is missing.  Only set if
        # missing_ok is True AND attr is missing.
        self._pass_by_missing = False

        self.missing_ok = config.get(MISSINOK_OPT)
        self.attr = config.get(ATTR_OPT)

    @property
    def attr(self):
        return self._attr

    @attr.setter
    def attr(self, value):
        try:
            if not value:
                raise InvalidCheckValue("{}: \"{}\" setting is missing.".format(self.checkname,
                                                                                ATTR_OPT))

            self._attr = self.policy.lookup_typeattr(value)
            self._pass_by_missing = False

        except InvalidType as e:
            if not self.missing_ok:
                raise InvalidCheckValue("{}: attr setting error: {}".format(
                    self.checkname, e)) from e

            self._attr = value
            self._pass_by_missing = True

    @property
    def missing_ok(self):
        return self._missing_ok

    @missing_ok.setter
    def missing_ok(self, value):
        self._missing_ok = config_bool_value(value)

        if self._missing_ok and isinstance(self.attr, str):
            # attr is only a string if it doesn't exist.
            self._pass_by_missing = True
        else:
            self._pass_by_missing = False

    def run(self) -> List:
        self.log.info("Checking type attribute {} is empty.".format(self.attr))

        failures = []

        if self._pass_by_missing:
            self.log_info("    {} does not exist.".format(self.attr))
        else:
            self.output.write("Member types of {}:\n".format(self.attr))

            types = sorted(self.attr.expand())
            if types:
                for type_ in types:
                    self.log_fail(type_.name)
                    failures.append(type_)
            else:
                self.log_ok("    <empty>")

        self.log.debug("{} failure(s)".format(failures))
        return failures



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


__all__ = [
    "__version__",
    "__author__",
    "__copyright__",
]

__version__ = "36.0.1"

__author__ = "The Python Cryptographic Authority and individual contributors"
__copyright__ = "Copyright 2013-2021 {}".format(__author__)



# Copyright 2015, Tresys Technology, LLC
# Copyright 2018, Chris PeBenito <pebenito@ieee.org>
#
# SPDX-License-Identifier: LGPL-2.1-only
#
from collections import defaultdict
from typing import NamedTuple, Set

from ..policyrep import Role, Type

from .descriptors import DiffResultDescriptor
from .difference import Difference, SymbolWrapper
from .typing import SymbolCache
from .types import type_wrapper_factory

_roles_cache: SymbolCache[Role] = defaultdict(dict)


class ModifiedRole(NamedTuple):

    """Difference details for a modified role."""

    added_types: Set[Type]
    removed_types: Set[Type]
    matched_types: Set[Type]


def role_wrapper_factory(role: Role) -> SymbolWrapper[Role]:
    """
    Wrap roles from the specified policy.

    This caches results to prevent duplicate wrapper
    objects in memory.
    """
    try:
        return _roles_cache[role.policy][role]
    except KeyError:
        r = SymbolWrapper(role)
        _roles_cache[role.policy][role] = r
        return r


class RolesDifference(Difference):

    """Determine the difference in roles between two policies."""

    added_roles = DiffResultDescriptor("diff_roles")
    removed_roles = DiffResultDescriptor("diff_roles")
    modified_roles = DiffResultDescriptor("diff_roles")

    def diff_roles(self) -> None:
        """Generate the difference in roles between the policies."""

        self.log.info(
            "Generating role differences from {0.left_policy} to {0.right_policy}".format(self))

        self.added_roles, self.removed_roles, matched_roles = self._set_diff(
            (role_wrapper_factory(r) for r in self.left_policy.roles()),
            (role_wrapper_factory(r) for r in self.right_policy.roles()))

        self.modified_roles = dict()

        for left_role, right_role in matched_roles:
            # Criteria for modified roles
            # 1. change to type set, or
            # 2. change to attribute set (not implemented)
            added_types, removed_types, matched_types = self._set_diff(
                (type_wrapper_factory(t) for t in left_role.types()),
                (type_wrapper_factory(t) for t in right_role.types()))

            if added_types or removed_types:
                self.modified_roles[left_role] = ModifiedRole(added_types,
                                                              removed_types,
                                                              matched_types)

    #
    # Internal functions
    #
    def _reset_diff(self) -> None:
        """Reset diff results on policy changes."""
        self.log.debug("Resetting role differences")
        self.added_roles = None
        self.removed_roles = None
        self.modified_roles = None



from __future__ import print_function
import contextlib
import datetime
import errno
import logging
import os
import time
import uuid
import traceback as _traceback

from systemd import journal, id128
from systemd.journal import _make_line

import pytest

TEST_MID = uuid.UUID('8441372f8dca4ca98694a6091fd8519f')
TEST_MID2 = uuid.UUID('8441370000000000000000001fd85000')

class MockSender:
    def __init__(self):
        self.buf = []

    def send(self, MESSAGE, MESSAGE_ID=None,
             CODE_FILE=None, CODE_LINE=None, CODE_FUNC=None,
             **kwargs):
        args = ['MESSAGE=' + MESSAGE]

        if MESSAGE_ID is not None:
            id = getattr(MESSAGE_ID, 'hex', MESSAGE_ID)
            args.append('MESSAGE_ID=' + id)

        if CODE_LINE is CODE_FILE is CODE_FUNC is None:
            CODE_FILE, CODE_LINE, CODE_FUNC = _traceback.extract_stack(limit=2)[0][:3]
        if CODE_FILE is not None:
            args.append('CODE_FILE=' + CODE_FILE)
        if CODE_LINE is not None:
            args.append('CODE_LINE={:d}'.format(CODE_LINE))
        if CODE_FUNC is not None:
            args.append('CODE_FUNC=' + CODE_FUNC)

        args.extend(_make_line(key, val) for key, val in kwargs.items())
        self.buf.append(args)

@contextlib.contextmanager
def skip_oserror(code):
    try:
        yield
    except (OSError, IOError) as e:
        if e.errno == code:
            pytest.skip()
        raise

@contextlib.contextmanager
def skip_valueerror():
    try:
        yield
    except ValueError:
        pytest.skip()

def test_priorities():
    p = journal.JournalHandler.map_priority

    assert p(logging.NOTSET)       == journal.LOG_DEBUG
    assert p(logging.DEBUG)        == journal.LOG_DEBUG
    assert p(logging.DEBUG - 1)    == journal.LOG_DEBUG
    assert p(logging.DEBUG + 1)    == journal.LOG_INFO
    assert p(logging.INFO - 1)     == journal.LOG_INFO
    assert p(logging.INFO)         == journal.LOG_INFO
    assert p(logging.INFO + 1)     == journal.LOG_WARNING
    assert p(logging.WARN - 1)     == journal.LOG_WARNING
    assert p(logging.WARN)         == journal.LOG_WARNING
    assert p(logging.WARN + 1)     == journal.LOG_ERR
    assert p(logging.ERROR - 1)    == journal.LOG_ERR
    assert p(logging.ERROR)        == journal.LOG_ERR
    assert p(logging.ERROR + 1)    == journal.LOG_CRIT
    assert p(logging.FATAL)        == journal.LOG_CRIT
    assert p(logging.CRITICAL)     == journal.LOG_CRIT
    assert p(logging.CRITICAL + 1) == journal.LOG_ALERT


def test_journalhandler_init_exception():
    kw = {' X  ':3}
    with pytest.raises(ValueError):
        journal.JournalHandler(**kw)

def test_journalhandler_init():
    kw = {'X':3, 'X3':4}
    journal.JournalHandler(logging.INFO, **kw)

def test_journalhandler_info():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)

    sender = MockSender()
    kw = {'X':3, 'X3':4, 'sender_function': sender.send}
    handler = journal.JournalHandler(logging.INFO, **kw)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert 'X=3' in sender.buf[0]
    assert 'X3=4' in sender.buf[0]

def test_journalhandler_no_message_id():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)
    sender = MockSender()
    handler = journal.JournalHandler(logging.INFO, sender_function=sender.send)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert all(not m.startswith('MESSAGE_ID=') for m in sender.buf[0])

def test_journalhandler_message_id_on_handler():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)
    sender = MockSender()
    handler = journal.JournalHandler(logging.INFO, sender_function=sender.send,
                                     MESSAGE_ID=TEST_MID)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert 'MESSAGE_ID=' + TEST_MID.hex in sender.buf[0]

def test_journalhandler_message_id_on_handler_hex():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)
    sender = MockSender()
    handler = journal.JournalHandler(logging.INFO, sender_function=sender.send,
                                     MESSAGE_ID=TEST_MID.hex)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert 'MESSAGE_ID=' + TEST_MID.hex in sender.buf[0]

def test_journalhandler_message_id_on_message():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)
    record.__dict__['MESSAGE_ID'] = TEST_MID2
    sender = MockSender()
    handler = journal.JournalHandler(logging.INFO, sender_function=sender.send,
                                     MESSAGE_ID=TEST_MID)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert 'MESSAGE_ID=' + TEST_MID2.hex in sender.buf[0]

def test_journalhandler_message_id_on_message_hex():
    record = logging.LogRecord('test-logger', logging.INFO, 'testpath', 1, 'test', None, None)
    record.__dict__['MESSAGE_ID'] = TEST_MID2.hex
    sender = MockSender()
    handler = journal.JournalHandler(logging.INFO, sender_function=sender.send,
                                     MESSAGE_ID=TEST_MID)
    handler.emit(record)
    assert len(sender.buf) == 1
    assert 'MESSAGE_ID=' + TEST_MID2.hex in sender.buf[0]

def test_reader_init_flags():
    j1 = journal.Reader()
    j2 = journal.Reader(journal.LOCAL_ONLY)
    j3 = journal.Reader(journal.RUNTIME_ONLY)
    j4 = journal.Reader(journal.SYSTEM_ONLY)
    j5 = journal.Reader(journal.LOCAL_ONLY | journal.RUNTIME_ONLY | journal.SYSTEM_ONLY)
    j6 = journal.Reader(0)

def test_reader_os_root(tmpdir):
    with pytest.raises(ValueError):
        journal.Reader(journal.OS_ROOT)
    with skip_valueerror():
        j1 = journal.Reader(path=tmpdir.strpath,
                            flags=journal.OS_ROOT)
    with skip_valueerror():
        j2 = journal.Reader(path=tmpdir.strpath,
                            flags=journal.OS_ROOT | journal.CURRENT_USER)
    j3 = journal.Reader(path=tmpdir.strpath,
                        flags=journal.OS_ROOT | journal.SYSTEM_ONLY)

def test_reader_init_path(tmpdir):
    j1 = journal.Reader(path=tmpdir.strpath)
    journal.Reader(0, path=tmpdir.strpath)

    j2 = journal.Reader(path=tmpdir.strpath)
    journal.Reader(path=tmpdir.strpath)

def test_reader_init_path_invalid_fd():
    with pytest.raises(OSError):
        journal.Reader(0, path=-1)

def test_reader_init_path_nondirectory_fd():
    with pytest.raises(OSError):
        journal.Reader(0, path=0)

def test_reader_init_path_fd(tmpdir):
    fd = os.open(tmpdir.strpath, os.O_RDONLY)

    with skip_oserror(errno.ENOSYS):
        j1 = journal.Reader(path=fd)
    assert list(j1) == []

    with skip_valueerror():
        j2 = journal.Reader(journal.SYSTEM, path=fd)
    assert list(j2) == []

    j3 = journal.Reader(journal.CURRENT_USER, path=fd)
    assert list(j3) == []

def test_reader_as_cm(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        assert not j.closed
    assert j.closed
    # make sure that operations on the Reader fail
    with pytest.raises(OSError):
        next(j)

def test_reader_messageid_match(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        j.messageid_match(id128.SD_MESSAGE_JOURNAL_START)
        j.messageid_match(id128.SD_MESSAGE_JOURNAL_STOP.hex)

def test_reader_this_boot(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        j.this_boot()
        j.this_boot(TEST_MID)
        j.this_boot(TEST_MID.hex)

def test_reader_this_machine(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        j.this_machine()
        j.this_machine(TEST_MID)
        j.this_machine(TEST_MID.hex)

def test_reader_query_unique(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        with skip_oserror(errno.ENOSYS):
            ans = j.query_unique('FOOBAR')
    assert isinstance(ans, set)
    assert ans == set()

def test_reader_enumerate_fields(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        with skip_oserror(errno.ENOSYS):
            ans = j.enumerate_fields()
    assert isinstance(ans, set)
    assert ans == set()

def test_reader_has_runtime_files(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        with skip_oserror(errno.ENOSYS):
            ans = j.has_runtime_files()
    assert ans is False

def test_reader_has_persistent_files(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)
    with j:
        with skip_oserror(errno.ENOSYS):
            ans = j.has_runtime_files()
    assert ans is False

def test_reader_converters(tmpdir):
    converters = {'xxx' : lambda arg: 'yyy'}
    j = journal.Reader(path=tmpdir.strpath, converters=converters)

    val = j._convert_field('xxx', b'abc')
    assert val == 'yyy'

    val = j._convert_field('zzz', b'\200\200')
    assert val == b'\200\200'

def test_reader_convert_entry(tmpdir):
    converters = {'x1' : lambda arg: 'yyy',
                  'x2' : lambda arg: 'YYY'}
    j = journal.Reader(path=tmpdir.strpath, converters=converters)

    val = j._convert_entry({'x1' : b'abc',
                            'y1' : b'\200\200',
                            'x2' : [b'abc', b'def'],
                            'y2' : [b'\200\200', b'\200\201']})
    assert val == {'x1' : 'yyy',
                   'y1' : b'\200\200',
                   'x2' : ['YYY', 'YYY'],
                   'y2' : [b'\200\200', b'\200\201']}

def test_seek_realtime(tmpdir):
    j = journal.Reader(path=tmpdir.strpath)

    now = time.time()
    j.seek_realtime(now)

    j.seek_realtime(12345)

    long_ago = datetime.datetime(1970, 5, 4)
    j.seek_realtime(long_ago)

def test_journal_stream():
    # This will fail when running in a bare chroot without /run/systemd/journal/stdout
    with skip_oserror(errno.ENOENT):
        stream = journal.stream('test_journal.py')

    res = stream.write('message...\n')
    assert res in (11, None) # Python2 returns None

    print('printed message...', file=stream)



# Copyright 2020, Microsoft Corporation
#
# SPDX-License-Identifier: LGPL-2.1-only
#


def config_bool_value(value) -> bool:
    """Convert a boolean configuration value."""

    if isinstance(value, str):
        if value and value.strip().lower() in ("yes", "true", "1"):
            return True

        return False

    return bool(value)



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


import abc
import typing

from cryptography import utils
from cryptography.exceptions import UnsupportedAlgorithm, _Reasons
from cryptography.hazmat.primitives._cipheralgorithm import (
    BlockCipherAlgorithm,
    CipherAlgorithm,
)


class Mode(metaclass=abc.ABCMeta):
    @abc.abstractproperty
    def name(self) -> str:
        """
        A string naming this mode (e.g. "ECB", "CBC").
        """

    @abc.abstractmethod
    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None:
        """
        Checks that all the necessary invariants of this (mode, algorithm)
        combination are met.
        """


class ModeWithInitializationVector(metaclass=abc.ABCMeta):
    @abc.abstractproperty
    def initialization_vector(self) -> bytes:
        """
        The value of the initialization vector for this mode as bytes.
        """


class ModeWithTweak(metaclass=abc.ABCMeta):
    @abc.abstractproperty
    def tweak(self) -> bytes:
        """
        The value of the tweak for this mode as bytes.
        """


class ModeWithNonce(metaclass=abc.ABCMeta):
    @abc.abstractproperty
    def nonce(self) -> bytes:
        """
        The value of the nonce for this mode as bytes.
        """


class ModeWithAuthenticationTag(metaclass=abc.ABCMeta):
    @abc.abstractproperty
    def tag(self) -> typing.Optional[bytes]:
        """
        The value of the tag supplied to the constructor of this mode.
        """


def _check_aes_key_length(self, algorithm):
    if algorithm.key_size > 256 and algorithm.name == "AES":
        raise ValueError(
            "Only 128, 192, and 256 bit keys are allowed for this AES mode"
        )


def _check_iv_length(self, algorithm):
    if len(self.initialization_vector) * 8 != algorithm.block_size:
        raise ValueError(
            "Invalid IV size ({}) for {}.".format(
                len(self.initialization_vector), self.name
            )
        )


def _check_nonce_length(nonce: bytes, name: str, algorithm) -> None:
    if len(nonce) * 8 != algorithm.block_size:
        raise ValueError(
            "Invalid nonce size ({}) for {}.".format(len(nonce), name)
        )


def _check_iv_and_key_length(self, algorithm):
    _check_aes_key_length(self, algorithm)
    _check_iv_length(self, algorithm)


class CBC(Mode, ModeWithInitializationVector):
    name = "CBC"

    def __init__(self, initialization_vector: bytes):
        utils._check_byteslike("initialization_vector", initialization_vector)
        self._initialization_vector = initialization_vector

    @property
    def initialization_vector(self) -> bytes:
        return self._initialization_vector

    validate_for_algorithm = _check_iv_and_key_length


class XTS(Mode, ModeWithTweak):
    name = "XTS"

    def __init__(self, tweak: bytes):
        utils._check_byteslike("tweak", tweak)

        if len(tweak) != 16:
            raise ValueError("tweak must be 128-bits (16 bytes)")

        self._tweak = tweak

    @property
    def tweak(self) -> bytes:
        return self._tweak

    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None:
        if algorithm.key_size not in (256, 512):
            raise ValueError(
                "The XTS specification requires a 256-bit key for AES-128-XTS"
                " and 512-bit key for AES-256-XTS"
            )


class ECB(Mode):
    name = "ECB"

    validate_for_algorithm = _check_aes_key_length


class OFB(Mode, ModeWithInitializationVector):
    name = "OFB"

    def __init__(self, initialization_vector: bytes):
        utils._check_byteslike("initialization_vector", initialization_vector)
        self._initialization_vector = initialization_vector

    @property
    def initialization_vector(self) -> bytes:
        return self._initialization_vector

    validate_for_algorithm = _check_iv_and_key_length


class CFB(Mode, ModeWithInitializationVector):
    name = "CFB"

    def __init__(self, initialization_vector: bytes):
        utils._check_byteslike("initialization_vector", initialization_vector)
        self._initialization_vector = initialization_vector

    @property
    def initialization_vector(self) -> bytes:
        return self._initialization_vector

    validate_for_algorithm = _check_iv_and_key_length


class CFB8(Mode, ModeWithInitializationVector):
    name = "CFB8"

    def __init__(self, initialization_vector: bytes):
        utils._check_byteslike("initialization_vector", initialization_vector)
        self._initialization_vector = initialization_vector

    @property
    def initialization_vector(self) -> bytes:
        return self._initialization_vector

    validate_for_algorithm = _check_iv_and_key_length


class CTR(Mode, ModeWithNonce):
    name = "CTR"

    def __init__(self, nonce: bytes):
        utils._check_byteslike("nonce", nonce)
        self._nonce = nonce

    @property
    def nonce(self) -> bytes:
        return self._nonce

    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None:
        _check_aes_key_length(self, algorithm)
        _check_nonce_length(self.nonce, self.name, algorithm)


class GCM(Mode, ModeWithInitializationVector, ModeWithAuthenticationTag):
    name = "GCM"
    _MAX_ENCRYPTED_BYTES = (2 ** 39 - 256) // 8
    _MAX_AAD_BYTES = (2 ** 64) // 8

    def __init__(
        self,
        initialization_vector: bytes,
        tag: typing.Optional[bytes] = None,
        min_tag_length: int = 16,
    ):
        # OpenSSL 3.0.0 constrains GCM IVs to [64, 1024] bits inclusive
        # This is a sane limit anyway so we'll enforce it here.
        utils._check_byteslike("initialization_vector", initialization_vector)
        if len(initialization_vector) < 8 or len(initialization_vector) > 128:
            raise ValueError(
                "initialization_vector must be between 8 and 128 bytes (64 "
                "and 1024 bits)."
            )
        self._initialization_vector = initialization_vector
        if tag is not None:
            utils._check_bytes("tag", tag)
            if min_tag_length < 4:
                raise ValueError("min_tag_length must be >= 4")
            if len(tag) < min_tag_length:
                raise ValueError(
                    "Authentication tag must be {} bytes or longer.".format(
                        min_tag_length
                    )
                )
        self._tag = tag
        self._min_tag_length = min_tag_length

    @property
    def tag(self) -> typing.Optional[bytes]:
        return self._tag

    @property
    def initialization_vector(self) -> bytes:
        return self._initialization_vector

    def validate_for_algorithm(self, algorithm: CipherAlgorithm) -> None:
        _check_aes_key_length(self, algorithm)
        if not isinstance(algorithm, BlockCipherAlgorithm):
            raise UnsupportedAlgorithm(
                "GCM requires a block cipher algorithm",
                _Reasons.UNSUPPORTED_CIPHER,
            )
        block_size_bytes = algorithm.block_size // 8
        if self._tag is not None and len(self._tag) > block_size_bytes:
            raise ValueError(
                "Authentication tag cannot be more than {} bytes.".format(
                    block_size_bytes
                )
            )
